/************************************************************
 * libdconf is (C) CopyRight PTlink IRC Software 1999-2005  *
 *                 http://software.pt-link.net              *
 * This program is distributed under GNU Public License     *
 * Please read the file COPYING for copyright information.  *
 ************************************************************

  Description: change item code for dconf parsing library

 *  $Id: change.c,v 1.1.1.1 2005/08/27 15:45:07 jpinto Exp $
*/

#include "stdinc.h"
#include "dconf.h"

void dconf_log(const char* fmt, ...);

/*
 * Returns time (in seconds) from time string (ending with s/m/h/d/M)
 */
static int time_str(char *s)
{
    int amount;
  
    amount = strtol(s, (char**) &s, 10);
    if (*s) {
        switch (*s) {
            case 's': return amount;
            case 'm': return amount*60;
            case 'h': return amount*3600;
            case 'd': return amount*86400;
            case 'M': return amount*30*86400;
            default : return -1;
        }
    } else {
        return amount;
    }
}

/*
 * dconf_change_item
 *
 * inputs       - item name, new value, change action
 * output       - change success
 * side effects - log any possible problem with the new setting
 */
int  dconf_change_item(char *name, char* value, int caction) 
  {
    int tmpval;
	
    int i = dconf_find_item(name);

	
    if(i<0)
	return (UNKNOWN_ITEM);
	  	  
     while(isspace(*value)) 
	  ++value;

	/* Let's remove the surrounding "" */
	if ((*value) == '\"') 
		*(value++)=' ';	
			  
	if (*value && (value[strlen(value)-1] == '\"') )
		value[strlen(value)-1]='\0';

	if(dconf_items[i].type == CT_FUNC) /* we must execute it */
	  {
	     ((int (*)(char *)) dconf_items[i].vptr)(value);
	     return 0;
	  }
	  
	if ((caction!=CA_READ) && (dconf_items[i].flags & CF_REQ) && (value[0]=='\0')) /* cannot undefine */
		return (LOCKED_ITEM);
	  	  
    if (dconf_items[i].flags & CF_DEFINED) 
	  { /* this value is already defined  */
		if (caction==CA_READ) 
		  {
            		dconf_log("Ignoring duplicated item %s", name);		
            		return 0;
		  }	
		else 
		  {
	  		if ((dconf_items[i].flags & CF_LOCKED)) /* item is locked */
			  return (LOCKED_ITEM);
	  		else 
			  {
			    if (dconf_items[i].type==CT_STRING) 
	  			  free( *(char**) dconf_items[i].vptr);
	  		  }	    
		  }
  	  }
	
    switch(dconf_items[i].type) 
	  {	
		case CT_DESC:
		break;		
		  
		case CT_STRING:	
			
		  /* Let's remove the surrounding "" */
		  if ((*value) == '\"') 
			*(value++)=' ';	
				  
		  if (*value && (value[strlen(value)-1] == '\"') )
			value[strlen(value)-1]='\0';
			
		  if(value[0]=='\0')
			{
	  	  	  *(char **)dconf_items[i].vptr = NULL;
			  dconf_items[i].flags &= ~CF_DEFINED;
			  return 1;
			}
		  else /* save the new data */
	  		*(char **)dconf_items[i].vptr = (value[0]) ? strdup(value) : NULL;
	  	break;
		
		case CT_SET:
		  if ( strcasecmp(value,"yes") == 0)
			*(int*) dconf_items[i].vptr = -1;
		  else if ( strcasecmp(value,"no") == 0)
			*(int*) dconf_items[i].vptr = 0;
		  else 
		  {
		    return (INV_SET_ITEM);
		  }		  
		break;
			
		case CT_TIME:
	  	  tmpval = time_str(value);
	  	  if (tmpval<=0) 
			{ 
    		  return (INV_TIME_ITEM);
	  		} 			  
	  	  else *(int*) dconf_items[i].vptr = tmpval;
		  dconf_items[i].flags |= CF_DEFINED;
		break;
		  
		default:
		  *(int*) dconf_items[i].vptr = atoi(value);
  	  }
	    	
	dconf_items[i].flags |= CF_DEFINED;	
    return 1;
  }
