/************************************************************
 * libdconf is (C) CopyRight PTlink IRC Software 1999-2004  *
 *                 http://software.pt-link.net              *
 * This program is distributed under GNU Public License     *
 * Please read the file COPYING for copyright information.  *
 ************************************************************
 
  Description: dconf parsing library header file

*  $Id: dconf.h,v 1.1.1.1 2005/08/27 15:45:07 jpinto Exp $
*/

#define DCONF_MAX 	128 	/* maximum dconf settings we can handle */

/* Change Actions */
#define CA_READ         0       /* reading dconf */
#define CA_REHASH       1       /* rehashing dconf */
#define CA_SET          2       /* run-time changing value */

/* Configuration item types */
#define CT_DESC         0       /* Description */
#define CT_STRING       1       /* string */
#define CT_TIME         2       /* time */
#define CT_SET          3       /* set (YES/NO) */
#define CT_INT          4       /* integer */
#define CT_FUNC		5	/* function */

/* change item return errors */
#define UNKNOWN_ITEM    -1
#define LOCKED_ITEM     -2
#define INV_SET_ITEM    -3
#define INV_TIME_ITEM   -4

/* Configuration item flags */
#define CF_REQ          0x0001  /* Required */
#define CF_LOCKED       0x0002  /* cannot be changed at runtime */
#define CF_DEFINED      0x0004  /* value is defined */

typedef struct {
    char *name;
    int  type;
    void *vptr;
    int flags;
} ConfItem;



/* functions */

extern int dconf_init(int logfd);
void  dconf_add(char* name, int type, int flags, void* vptr, char* def);
void  dconf_del(char* name);
extern int dconf_read(char *fname, int rehashing);
extern int dconf_add_and_read(ConfItem [], char *fname);
extern void dconf_remove_items(ConfItem dconfitems[]);
extern int dconf_change_item(char *name, char* value, int caction);
extern int dconf_find_item(char *name);
extern int dconf_check(void);
extern char* dconf_item_str(int i);
extern int dconf_value(char *item);
extern char* dconf_valuestr(char *item);


ConfItem dconf_items[DCONF_MAX];

/* some usefull macros */
#define DCONF_DESC(x) { (x), CT_DESC, NULL, 0 },
#define DCONF_STRING(x,y) { #x, CT_STRING, &(x), (y) },
#define DCONF_INT(x,y) { #x, CT_INT, &(x), (y) },
#define DCONF_SET(x,y) { #x, CT_SET, &(x), (y) },
#define DCONF_TIME(x,y) { #x, CT_TIME, &(x), (y) },
#define DCONF_FUNC(x) { #x, CT_FUNC, (void*) &(x), 0 },

#define DCONF_GET_VALUE(x) (x) = dconf_value(#x);
#define DCONF_GET_STRVALUE(x) (x) = dconf_valuestr(#x);

