/************************************************************
 * libdconf is (C) CopyRight PTlink IRC Software 1999-2004  *
 *                 http://software.pt-link.net              *
 * This program is distributed under GNU Public License     *
 * Please read the file COPYING for copyright information.  *
 ************************************************************

  Description: read code for dconf parsing library

*  $Id: read.c,v 1.1.1.1 2005/08/27 15:45:09 jpinto Exp $   
*/

#include "stdinc.h"

#ifdef HAVE_LIBGEN_H
#include <libgen.h>
#endif

#include "dconf.h"

void dconf_log(const char* fmt, ...);

/*
 * dconf_read
 *
 * inputs       - .conf filename and rehash mode
 * output       - number of errors during .conf reading
 * side effects - log any possible problems
 */
int dconf_read(char *fname, int rehashing)
  {
    char line[512];	/* buffer to read .conf line */
    char basedir[128];
    char fn[128];
    FILE *confile;
    char* item;
    char* value;
    char* auxptr;
    int	i;
    int errors = 0; /* errors found reading conf */
    int caction;    /* change action for change_* */

    if(rehashing)
      {
        dconf_log("Reading configuration file %s (during rehash)", fname);
        caction = CA_REHASH;
      }
    else
      {
        dconf_log("Reading configuration file %s", fname);
        caction = CA_READ;
      };     
    auxptr = strdup(fname);
    strncpy(basedir, dirname(auxptr), sizeof(basedir));
    free(auxptr);
    
    confile = fopen(fname,"rt");      
    if(confile==NULL) 
      {
        dconf_log("Unable to open file %s", fname);
        return -2;
      }    
    while (!feof(confile)) 
      {
      
        line[0] = '\0';
	if(fgets(line, sizeof(line), confile)==NULL) /* nothing to parse */
	  break;
	  
	auxptr = line;
		
	while(isspace(*auxptr))
	  ++auxptr;
	item = strtok(auxptr,"\t\n ");		
	if( !item || (item[0] == '\0') || (item[0] == '#'))
	  continue;
		  
	value = strtok(NULL,"\r\n");

		if (!value) 
		  {
	        	dconf_log("Missing value for item %s reading %s", item, fname);
			continue;
		  }
		
		while(isspace(*value)) 
		  ++value;
		
		if ( strcmp(item,".include") == 0 )
		  {
		     snprintf(fn, sizeof(fn), "%s/%s", basedir, value); 
		     if (dconf_read(fn, rehashing) == -1)
		       {
		         dconf_log("Error opening include file %s",value);
		       }
		     continue;
		  }

		i = dconf_change_item(item, value, caction);
		
		switch(i) 
          	{
              case UNKNOWN_ITEM: 
              dconf_log("Ignoring unknown item %s", item);
              break;
            case LOCKED_ITEM: break;
            case INV_SET_ITEM: 
              dconf_log("Invalid value (%s) on .conf item %s, please use Yes/No", value, item);
              break;
            case INV_TIME_ITEM:
              dconf_log("Invalid time format on .conf item %s", item);
              break;
          }
		if(i<0 && (i!=LOCKED_ITEM))
		  ++errors;
	  }
	  
	fclose(confile);
	if(errors)
	  dconf_log("Closed %s: had %i error(s)", fname, errors);
/* Noisy log	  
	else
          dconf_log("Closed %s: No errors", fname);
*/       
    return errors;	  
  }
  
/*
   dconf_add_and_read
   	This function will add a group items, read from a conf file 
   	and returns the check result.
 */
int dconf_add_and_read(ConfItem dconfitems[], char *fname)
{
  int i = 0;
  ConfItem *dc;
  int res;
    
  /* add items to the global dconf array */
  while((dc = &dconfitems[i++]) && dc->name)
    dconf_add(dc->name, dc->type, dc->flags, dc->vptr, NULL);
    
  /* read the dconf file */
  res = dconf_read(fname, 0);
  if(res >= 0) /* check for required items */
    res = dconf_check();
  
  if(res < 0) /* rollback configuration items add */
    {
      i = 0;
      while((dc = &dconfitems[i++]) && dc->name)
        dconf_del(dc->name);
    }
  return res;
}
