/**********************************************************************
 * PTlink IRC Services is (C) CopyRight PTlink IRC Software 1999-2005 *
 *                     http://software.pt-link.net                    *
 * This program is distributed under GNU Public License               *
 * Please read the file COPYING for copyright information.            *
 **********************************************************************

  Description: shows last registered channels

 *  $Id: cs_lastreg.c,v 1.1.1.1 2005/08/27 15:44:37 jpinto Exp $
*/

#include "stdinc.h"
#include "dconf.h"
#include "modules.h"
#include "ircservice.h"
#include "suser.h"
#include "my_sql.h"
#include "strhand.h"
#include "lang.h"
#include "chanserv.h"
#include "modevents.h"
#include "lang/cs_lastreg.lh"

SVS_Module mod_info =
 /* module, version, description */
{"cs_lastreg", "1.1",  "last registered channels" };
/* Change Log
  1.1 -  0000257: unload module cs_lastreg crashes
*/
  
/** functions and events we require **/
ServiceUser* (*chanserv_suser)(void);
int e_chan_register;

Module_Function mod_requires[] = 
{
  MOD_FUNC(chanserv_suser)
  MOD_FUNC(e_chan_register)
  {NULL}
};

/** functions and events we provide **/
/* void my_function(void); */

Module_Function mod_provides[] = 
{
  {NULL}
};

/** Internal functions declaration **/
int mod_load(char *param);
int mod_unload(char *param);
void ev_cs_lastreg_new_user(IRC_User* u, void *s);
int ev_cs_lastreg_chan_register(IRC_User *u, ChanRecord* cr);

    
/** Local dconf settings **/
int LastRegCount = 0;	/* how many channels we will show */

ConfItem mod_dconf[] = 
{
  DCONF_INT(LastRegCount, CF_REQ)
  { NULL }
};


/** Local variables **/
static ServiceUser* csu;
static char **last_reg_list;

/** load code **/
int mod_load(char *param)
{
  int i;
  MYSQL_RES* res;
  MYSQL_ROW row;

  csu = chanserv_suser();
  
  /* alocate memory to keep the chan list*/
  last_reg_list = malloc(LastRegCount * sizeof(char*));  
  res = sql_query("SELECT name FROM chanserv "
    "WHERE (flags & %d)=0 ORDER BY t_reg DESC "
    "LIMIT %d", CFL_PRIVATE, LastRegCount);
  i = 0;
  while((row = sql_next_row(res)) && i < LastRegCount)
    {
      last_reg_list[i++] = strdup(row[0]);
    }         
  sql_free(res);
  
  while(i < LastRegCount) /* just clear the remaining free items */
    last_reg_list[i++] = NULL;     
  
  /* Add event to dump list */ 
  irc_AddEvent(ET_NEW_USER, ev_cs_lastreg_new_user); /* new user */
  
  /* Add action to update list */
  mod_add_event_action(e_chan_register, (ActionHandler) ev_cs_lastreg_chan_register);
    
  return 0;
}
    
/** unload code **/
int mod_unload(char *param)
{
  int i;
  /* just free the list mem */
  for(i=0; i < LastRegCount; ++i)
    {
      FREE(last_reg_list[i]);
    }
  free(last_reg_list);
  irc_DelEvent(ET_NEW_USER, ev_cs_lastreg_new_user);
  return 0;
}
    
/** internal functions implementation starts here **/
void ev_cs_lastreg_new_user(IRC_User* u, void *s)
{
  int i;
  if(irc_IsUMode(u, UMODE_IDENTIFIED)) /* new user with +r from a netjoin */
    return;
  
  send_lang(u, csu->u, LAST_X_REGCHANS, LastRegCount);
  for(i=0; i < LastRegCount; ++i)
    if(last_reg_list[i])
      send_lang(u, csu->u, LAST_ITEM_X, last_reg_list[i]);
  send_lang(u, csu->u, LAST_TRAIL);
}

/* we need to update the last reg list here */
int ev_cs_lastreg_chan_register(IRC_User *u, ChanRecord* cr)
{
  int i;
  FREE(last_reg_list[LastRegCount-1]); /* free out item */
    
  for(i=LastRegCount-1; i > 0; --i)
    last_reg_list[i]=last_reg_list[i-1];
  last_reg_list[0] = strdup(cr->name); /* update the first item */  
  return 0;
}
    
/* End of Module */
const int MVERSION = MODULES_VERSION;
