/******************************************************************
 * PTlink Services is (C) CopyRight PTlink IRC Software 1999-2005 *
 *                http://software.pt-link.net                     *
 * This program is distributed under GNU Public License           *
 * Please read the file COPYING for copyright information.        *
 ******************************************************************
                                                                                
  File: cs_show.c
  Description: chanserv list command
                                                                                
 *  $Id: cs_show.c,v 1.1.1.1 2005/08/27 15:44:41 jpinto Exp $
*/
#include "chanserv.h"
#include "stdinc.h"
#include "ircservice.h"
#include "suser.h"
#include "dconf.h"
#include "s_log.h"
#include "lang.h"
#include "modules.h"
#include "chanrecord.h"
#include "strhand.h"
#include "ircsvs.h"
#include "my_sql.h"
#include "nsmacros.h" /* CHECK_IF_IDENTIFIED_NICK */
#include "nickserv.h"
#include "lang/nscommon.lh"
#include "lang/cs_show.lh"

SVS_Module mod_info =
/* module, version, description */
{"cs_show", "2.0", "chanserv list command" };

/* Change Log
  2.0 - 0000265: remove nickserv cache system
      - 0000281: no auth nick can't use chanserv
*/

/* external functions we need */
ServiceUser* (*chanserv_suser)(void);

Module_Function mod_requires[] =
{
  MOD_FUNC(chanserv_suser)
  { NULL }
};

/* internal functions */
int mod_load(char *param);
void mod_unload(void);

/* available commands from module */
void cs_show(IRC_User *s, IRC_User *u);

/* Local settings */
int ChanServNeedsAuth = 0;

/* Configuration load */
int get_local_dconf(void)
{
  DCONF_GET_VALUE(ChanServNeedsAuth)
  return 0;
}

/* Local variables */
ServiceUser* csu;

int mod_load(char *param)
{
  csu = chanserv_suser();  
  get_local_dconf();
  suser_add_cmd(csu, "SHOW", cs_show, SHOW_SUMMARY, SHOW_HELP);
  return 0;
}

void mod_unload(void)
{
  suser_del_mod_cmds(csu, &mod_info);
}

/* s = service the command was sent to
   u = user the command was sent from */
void cs_show(IRC_User *s, IRC_User *u)
{
  MYSQL_RES* res;
  MYSQL_ROW row;
  u_int32_t source_snid;
  int rowc = 0;
  
  CHECK_IF_IDENTIFIED_NICK
  if(ChanServNeedsAuth && !IsAuthenticated(u))
    send_lang(u, s, NEEDS_AUTH_NICK);
  else
  {
  /* List channels you are founder at */
    res = sql_query("SELECT name FROM chanserv WHERE founder=%d", 
	source_snid);
     row = sql_next_row(res);
     if(row) 
     {
      send_lang(u, s, SHOW_FOUNDER_HEADER);
      do 
      {
        ++rowc;
        send_lang(u, s, SHOW_ITEM_X, row[0]);
      } while((row = sql_next_row(res)));
      send_lang(u, s, SHOW_TAIL_X, rowc);
    }
    else
	send_lang(u, s, SHOW_FOUNDER_EMPTY);
    sql_free(res);  
    rowc = 0;
    /* List channels you are successor at */
    res = sql_query("SELECT name FROM chanserv WHERE successor=%d", 
	source_snid);
    row = sql_next_row(res);
    if(row) 
    {
      send_lang(u, s, SHOW_SUCCESSOR_HEADER);
      do 
      {
        ++rowc;
        send_lang(u, s, SHOW_ITEM_X, row[0]);
      } while((row = sql_next_row(res)));
      send_lang(u, s, SHOW_TAIL_X, rowc);
    }
    sql_free(res);

    rowc = 0;

    /* List channels you are successor at */
    res = sql_query("SELECT r.name, c.name, cr.message "
	"FROM cs_role r, chanserv c, cs_role_users cr "
	"WHERE cr.snid=%d AND r.rid=cr.rid AND c.scid=cr.scid", 
	source_snid);
    
    row = sql_next_row(res);
     if(row) 
     {
      send_lang(u, s, SHOW_ROLES_HEADER);
      do 
      {
        ++rowc;
        if(row[2])
          send_lang(u, s, SHOW_ROLES_ITEM_X_X_X, row[0], row[1], row[2]);
        else
          send_lang(u, s, SHOW_ROLES_ITEM_X_X, row[0], row[1]);
      } while((row = sql_next_row(res)));
      send_lang(u, s, SHOW_ROLES_TAIL_X, rowc);
     }
     sql_free(res);  
  }
}
/* End of Module */
const int MVERSION = MODULES_VERSION;
