/*****************************************************************
 * PTlink Services is (C) CopyRight PTlink IRC Software 1999-2004 *
 *                http://software.pt-link.net                     *
 * This program is distributed under GNU Public License          *
 * Please read the file COPYING for copyright information.       *
 *****************************************************************
                                                                                
  Description: chanserv unban command
                                                                                
 *  $Id: cs_unban.c,v 1.1.1.1 2005/08/27 15:44:47 jpinto Exp $
*/
#include "chanserv.h"
#include "stdinc.h"
#include "ircservice.h"
#include "path.h"
#include "suser.h"
#include "dconf.h"
#include "s_log.h"
#include "lang.h"
#include "modules.h"
#include "chanrecord.h"
#include "strhand.h"
#include "ircsvs.h"
#include "modevents.h"
#include "my_sql.h"
#include "log.h"
#include "cs_role.h"
#include "nsmacros.h"
#include "nickserv.h"
#include "lang/common.lh"
#include "lang/cscommon.lh"
#include "lang/nscommon.lh"
#include "lang/cs_unban.lh"

SVS_Module mod_info =
/* module, version, description */
{"cs_unban", "2.0", "chanserv unban command" };
/* Change Log
  2.0 - 0000265: remove nickserv cache system
      - 0000281: no auth nick can't use chanserv
*/

/* external functions we need */
ServiceUser* (*chanserv_suser)(void);
int* (*role_with_permission)(u_int32_t scid, u_int32_t snid, int permission);

Module_Function mod_requires[] =
{
  MOD_FUNC(chanserv_suser)
  MOD_FUNC(role_with_permission)
  { NULL }
};

/* Internal functions declaration */
int mod_install(char *param);
int mod_load(char *param);
void mod_unload(void);

/* core event handlers */

/* available commands from module */
void cs_unban(IRC_User *s, IRC_User *u);

ServiceUser* csu;
int cs_log;
int ChanServNeedsAuth = 0;

int get_local_dconf(void)
{
    DCONF_GET_VALUE(ChanServNeedsAuth)
    return 0;
}

int mod_load(char *param)
{

  csu = chanserv_suser();  
  get_local_dconf();
  suser_add_cmd(csu, "UNBAN", (void*) cs_unban, UNBAN_SUMMARY, UNBAN_HELP);    
  return 0;
}

void mod_unload(void)
{
  suser_del_mod_cmds(csu, &mod_info);
}

/* s = service the command was sent to
   u = user the command was sent from */
void cs_unban(IRC_User *s, IRC_User *u)
{
  u_int32_t source_snid;
  ChanRecord* cr;
  IRC_Chan* chan;
  char *chname;
  
  cr = NULL;
  chname = strtok(NULL, " ");  

  /* status validation */
  CHECK_IF_IDENTIFIED_NICK  
 
  /* syntax validation */ 
  if(ChanServNeedsAuth && !IsAuthenticated(u))
    send_lang(u, s, NEEDS_AUTH_NICK);
  else
  if(IsNull(chname))
    send_lang(u, s, UNBAN_SYNTAX);    
  /* check requirements */
  else if((chan = irc_FindChan(chname)) == NULL)
    send_lang(u,s, CHAN_X_IS_EMPTY, chname);
  else if((cr = chan->sdata) == NULL)
    send_lang(u, s, CHAN_X_NOT_REGISTERED, chname);    
  /* privileges validation */
  else if(role_with_permission(cr->scid, source_snid, P_UNBAN) == 0)
    send_lang(u, s, NO_UNBAN_PERM_ON_X, chname);
  /* execute operation */
  else
    {
      int r;
      r =  irc_ChanUnban(chan, u, chan->local_user ? chan->local_user : s);
      if(r == 0)
        send_lang(u, s, NO_BANS_ON_X, chname);
      else
        send_lang(u, s, REMOVED_X_BANS_ON_X, r, chname);
      irc_SvsJoin(u, s, chname);
    }

}

/* End of Module */
const int MVERSION = MODULES_VERSION;
