/**********************************************************************
 * PTlink IRC Services is (C) CopyRight PTlink IRC Software 1999-2005 *
 *                     http://software.pt-link.net                    *
 * This program is distributed under GNU Public License               *
 * Please read the file COPYING for copyright information.            *
 **********************************************************************

  Description: nickserv module

 *  $Id: nickserv.c,v 1.1.1.1 2005/08/27 15:44:25 jpinto Exp $
*/

#include "stdinc.h"
#include "strhand.h"
#include "nickserv.h"
#include "ircservice.h"
#include "s_log.h"
#include "log.h"
#include "suser.h"
#include "dconf.h"
#include "modules.h"
#include "path.h"
#include "lang.h"
#include "ircsvs.h"
#include "s_log.h"
#include "encrypt.h"
#include "modevents.h"
#include "my_sql.h"
#include "nsmacros.h"
#include "lang/common.lh"
#include "lang/nscommon.lh"
#include "lang/nickserv.lh"

SVS_Module mod_info = 
  {"nickserv", "5.0", "nickserv core module" };
/* Change Log
  5.0 - 0000305: foreign keys for data integrity 
        Added nickserv.5.sql changes
  4.0 - 0000276: nick password expire option
        0000272: move nickserv security info to a specific table
        0000265: remove nickserv cache system  
        Added nickserv.4.sql changes
  3.0 - Added nickserv.3.sql changes
        0000255: new field to store nick expire time
  2.0 - Added nickserv.2.sql changes
        0000218: NickDefaultOptions to set nick default options
        0000243: protected nick set option
        0000245: use snid to track previous identify  
  1.1 - ghost recognition was not setting +r with DisableNickSecurityCode
*/
          
#define DB_VERSION	5

/** functions and events we require **/
/* void (*FunctionPointer)(void);*/
int mysql_connection;
int e_expire;
Module_Function mod_requires[] =
{
  MOD_FUNC(e_expire) /* we need this to run the expire routines */
  MOD_FUNC(mysql_connection)
  /* MOD_FUNC(FunctionName)
  ...
  */
  {NULL}
};

/* functions we provide */
ServiceUser* nickserv_suser(void);
int e_nick_identify;
int e_nick_delete;
int e_nick_info;
int update_nick_online_info(IRC_User* u, u_int32_t snid, int lang);
int check_nick_security(u_int32_t snid, IRC_User *u, char* pass, char *email, int flags);
                                                                                
Module_Function mod_provides[] = {
  MOD_FUNC(nickserv_suser)
  MOD_FUNC(update_nick_online_info)
  MOD_FUNC(check_nick_security)
  MOD_FUNC(e_nick_identify)
  MOD_FUNC(e_nick_delete)  
  MOD_FUNC(e_nick_info)
  { NULL }
};

/* dconf settings we use */
char* NS_Nick;
char* NS_Username;
char* NS_Host;
char* NS_Info;
int DisableNickSecurityCode = 0; 
int ForceStrongPasswords;
char *NickProtectionPrefix;
int MaxProtectionNumber;
int NSMaxNickChange;
int MaxNicksPerMail;
char *NSLogChan;
int NSExpire = 0;
int AgeBonusPeriod = 0;
int AgeBonusValue = 0;
char* NickDefaultOptions = NULL;
int NickPassExpire = 0;

ConfItem mod_dconf[] = {
	DCONF_DESC("NickServ core configuration")
	DCONF_STRING(NS_Nick, CF_REQ)
	DCONF_STRING(NS_Username, CF_REQ)
	DCONF_STRING(NS_Host, CF_REQ)
	DCONF_STRING(NS_Info, CF_REQ)
	DCONF_STRING(NickProtectionPrefix, CF_REQ)
	DCONF_INT(MaxProtectionNumber, CF_REQ)
	DCONF_INT(NSMaxNickChange, 0)	
	DCONF_INT(MaxNicksPerMail, CF_REQ)
	DCONF_STRING(NSLogChan, 0)
	DCONF_TIME(NSExpire, 0)
        DCONF_TIME(AgeBonusPeriod, 0)
        DCONF_TIME(AgeBonusValue, 0)
        DCONF_STRING(NickDefaultOptions, 0)
        DCONF_TIME(NickPassExpire, 0)        
	{ NULL }
};

/* Local vars */
ServiceUser nsu;
int ns_log;

/* core events */
void ev_ns_new_user(IRC_User* u, void *null); /* needs to be fixed with IRC_Server */
void ev_ns_quit(IRC_User* u, char* reason);
void ev_ns_kill(IRC_User* u, IRC_User *killer);
void ev_ns_nick_change(IRC_User* u, char* lastnick);
void timer_ns_not_identifed(IRC_User* u, int tag);
int ev_ns_expire(void* dummy1, void* dummy2);
                          
/* commands */
void ns_unknown(IRC_User* s, IRC_User* t);

/* internal functions */
int mod_load(char *param);
void mod_unload(void);
void apply_prefix_nchange(IRC_User *u);
int valid_for_registration(char *nick);
void set_offline_info(IRC_User *u);
int sql_upgrade(int version, int post);
/* int update_nick_online_info(IRC_User* u, u_int32_t snid, int lang); */

int mod_load(char *param)
{

  if(NSExpire> 0 && NSExpire < 86400)
    {
      errlog(L_ERROR, "NSExpire is too low, minimum is 1 day!");
      return -6;
    }
    
  /* first try to open the log file */
  ns_log = log_open("nickserv","nickserv");
  
  if(ns_log<0)
    {
      errlog(L_ERROR, "Could not open nickserv log file!");
      return -1;
    }

  if(sql_check_inst_upgrade(mod_info.name, DB_VERSION, sql_upgrade) < 0)
    return -4;
    
  /* Create the nickserv client */
  nsu.u = irc_CreateLocalUser(NS_Nick, NS_Username, NS_Host, NS_Host,
    NS_Info,"+ro");
  /* NS should join the log chan */  
  if(NSLogChan)
    {
      IRC_Chan *chan;
      log_set_irc(ns_log, NS_Nick, NSLogChan);
      chan = irc_ChanJoin(nsu.u, NSLogChan, CU_MODE_ADMIN | CU_MODE_OP );
      irc_ChanMode(nsu.u, chan, "+Osnt");
    }
  /* Add unknown command handler */
  irc_AddUMsgEvent(nsu.u, "*", (void*) ns_unknown); /* any other msg handler */
  
  /* Add user events and register functions */
  irc_AddEvent(ET_NEW_USER, ev_ns_new_user); /* new user */
  irc_AddEvent(ET_NICK_CHANGE, ev_ns_nick_change); /* nick change */
  irc_AddEvent(ET_QUIT, ev_ns_quit); /* user quit */
  irc_AddEvent(ET_KILL, ev_ns_kill); /* user was killed */

  /* reset nick status */
  sql_query("UPDATE nickserv SET status=0");

  if(NSExpire == 0)
    stdlog(L_INFO, "NSExpire is not set, no nicks will expire");      
  else    
    mod_add_event_action(e_expire, (ActionHandler) ev_ns_expire);
  
  return 0;    
}

void
mod_unload(void)
{

  /* remove nickserv and all associated events */
  irc_QuitLocalUser(nsu.u, "Removing service");

  /* remove irc events */
  irc_DelEvent(ET_NEW_USER, ev_ns_new_user);
  irc_DelEvent(ET_NICK_CHANGE, ev_ns_nick_change);
  irc_DelEvent(ET_QUIT, ev_ns_quit);
  irc_DelEvent(ET_KILL, ev_ns_kill);
  lang_delete_assoc(); /* delete lang associations */
}

/* s = service the command was sent to
   u = user the command was sent from */
void ns_unknown(IRC_User* s, IRC_User* t)
{
  send_lang(t, s, UNKNOWN_COMMAND, irc_GetLastMsgCmd());
}


void timer_ns_not_identifed(IRC_User* u, int tag)
{ 
  
  if(NSMaxNickChange && u->guest_count >= NSMaxNickChange)
    {
      log_log(ns_log, mod_info.name,
        "Killing %s , too many nick changes", irc_UserMask(u));
      irc_Kill(u, nsu.u, "Too many guest nick changes");
      return;
    }
    apply_prefix_nchange(u);
}

/* new user was introduced
  Note: This is also called after a nick change for the new nick
*/
void ev_ns_new_user(IRC_User* u, void *null)
{
  u_int32_t snid = 0;
  u_int32_t flags = 0;
  int lang;
  char *ajoin = NULL;
  char *email = NULL;
  
  /* Set language according to the hostname */
  u->lang = lang_for_host(u->publichost);
  lang = u->lang;
  if( sql_singlequery("SELECT snid, flags, lang, ajoin, email FROM nickserv WHERE nick=%s",
    sql_str(irc_lower_nick(u->nick))) )
  {
    snid = sql_field_i(0);
    flags = sql_field_i(1);
    lang = sql_field_i(2);
    ajoin = sql_field(3);
    email = sql_field(4);    
  }
    
  if(snid == 0) /* nick not registered or record not found */
  {
    if(irc_IsUMode(u, UMODE_IDENTIFIED)) /* remove +r just to be safe */
      irc_SvsMode(u, nsu.u, "-r");    
    if(valid_for_registration(u->nick))
      send_lang(u, nsu.u, NICK_IS_NOT_REGISTERED);
    u->flags = 0;
    u->status = 0;
  }
  else if(flags & NFL_FORBIDDEN)
  {
    send_lang(u, nsu.u, NICK_IS_FORBIDDEN);
    apply_prefix_nchange(u);
  }    
  else if((u->use_snid == snid) || (u->req_snid == snid)) 
  { /* user already identifed for this nick */
    u->flags = flags;      
    check_nick_security(snid, u, NULL, email, flags);
    if((u->req_snid == snid) && ajoin)
      irc_SvsJoin(u, nsu.u, ajoin);    
    update_nick_online_info(u, snid, lang);
    mod_do_event(e_nick_identify, u, &snid);
    u->req_snid = 0;      
  }
  else if(irc_IsUMode(u, UMODE_IDENTIFIED)) /* nick was identified */
  {
    u->flags = flags;  
    update_nick_online_info(u, snid, lang);
    mod_do_event(e_nick_identify, u, &snid);      
  }
  else if(flags & NFL_PROTECTED)
  {  
    send_lang(u, nsu.u, NICK_IS_PROTECTED);
    apply_prefix_nchange(u);
  }
  else
  {
    u->flags = 0;
    u->status = 0;
    if(irc_IsUMode(u, UMODE_IDENTIFIED)) /* maybe nick was dropped ? */
      irc_SvsMode(u, nsu.u, "-r");
    send_lang(u, nsu.u, NICK_IS_REGISTERED);
    send_lang(u, nsu.u, CHANGE_IN_1M);
    irc_AddUTimerEvent(u, 60 , timer_ns_not_identifed, 0);
  }
}

void ev_ns_nick_change(IRC_User* u, char *lastnick)
{
  irc_CancelUserTimerEvents(u); /* we don't need events for changed nicks */
  set_offline_info(u);
  ev_ns_new_user(u, NULL);    /* Handle as usual nick logon */
}

void ev_ns_quit(IRC_User* u, char* reason)
{
  set_offline_info(u);
}

void ev_ns_kill(IRC_User* u, IRC_User *killer)
{
  set_offline_info(u);
}

/*
 Apply guest nick change (change nick to _nick-number)
 */ 
void apply_prefix_nchange(IRC_User *u)
{
  irc_SvsGuest(u, nsu.u, NickProtectionPrefix, MaxProtectionNumber);
}

/* check if nick can be registered */
int valid_for_registration(char *nick)
{
  static int nlen = 0;
  if(nlen==0)
    nlen = strlen(NickProtectionPrefix);
  if(ircncmp(NickProtectionPrefix, nick, nlen) == 0)
    return 0;
  return -1;
}

void set_offline_info(IRC_User *u)
{
  int i;
  
  if(u->snid)
    sql_execute("UPDATE nickserv SET status=0, t_seen=%d WHERE snid=%d",
      (int)irc_CurrentTime, u->snid);
  u->snid = 0;
  u->status = 0;
  u->flags = 0;

  for(i = 0; i < 16; ++i)
  {
    array_free(u->extra[i]);
    u->extra[i] = NULL;
  }
}

/* to return the nickserv client */
ServiceUser* nickserv_suser(void)
{
  return &nsu;
}

/* here we are going to use some raw mysql functions
 * because event actions for e_nick_delete may call sql_* functions
 * and override our query state
 */
int ev_ns_expire(void* dummy1, void* dummy2)
{
  time_t expire_start;
  MYSQL_RES* res;
  MYSQL_ROW row;
  int rowc = 0;

  /* expire code goes here */
  res = sql_query("SELECT snid, nick FROM nickserv "
      "WHERE (flags & %d = 0) AND (status & %d = 0) AND "
      "t_expire<%d", 
      NFL_NOEXPIRE | NFL_FORBIDDEN, NST_ONLINE, irc_CurrentTime);      

  if(res)
    rowc = mysql_num_rows(res);

  if(rowc == 0)
    {
      sql_free(res);
      return 0;
    }
  
  log_log(ns_log, mod_info.name, "Will expire %d nick(s)", rowc);
  expire_start = time(NULL);
    
  /* fetch the data */
  while((row = sql_next_row(res)))
    {
      IRC_User *u;
      u_int32_t snid = atoi(row[0]);
      u = irc_FindUser(row[1]);
      if(u && u->snid) /* nick is online and identified */
        {
          irc_SvsMode(u, nsu.u, "-r");
          u->snid = 0;
        }        
      log_log(ns_log, mod_info.name, "Expiring snid %d, nick %s", snid, row[1]);
      /* call related actions */
      mod_do_event(e_nick_delete, &snid, NULL);            
      /* and delete it */
      sql_execute("DELETE FROM nickserv WHERE snid=%d", snid);
    }
    
  sql_free(res);
  log_log(ns_log, mod_info.name, 
    "Expire routine terminated, took %s to expire %d nick(s)", 
    str_time(time(NULL) - expire_start), rowc);
  return 0;
}

/* this function takes care of sql upgrades */
int sql_upgrade(int version, int post)
{
  if(version == 3 && post)/* Upgrading to version 3, need to set t_expire */
  {
    log_log(ns_log, mod_info.name, "Updating t_expire");    
                    
    if(AgeBonusPeriod && AgeBonusValue)
      sql_execute("UPDATE nickserv "
        "SET t_expire=t_seen+%d+FLOOR((%d-t_reg)/%d)*%d",
        NSExpire, (int) time(NULL), AgeBonusPeriod, AgeBonusValue);
    else
      sql_execute("UPDATE nickserv "
        "SET t_expire=t_seen+%d,", NSExpire);
  }
  return 0;
}

int update_nick_online_info(IRC_User* u, u_int32_t snid, int lang)
{
  char sql_expire[64];

  /* update nick info */
  u->snid = snid;
  u->lang = lang;
  u->use_snid = snid;    
  u->status |= NST_ONLINE;
  if(AgeBonusPeriod && AgeBonusValue)
    snprintf(sql_expire, sizeof(sql_expire)-1, "%d+FLOOR((%d-t_reg)/%d)*%d",
      (int)irc_CurrentTime+NSExpire, (int)irc_CurrentTime, AgeBonusPeriod, AgeBonusValue);
  else
    snprintf(sql_expire, sizeof(sql_expire)-1,"%d",
      (int) irc_CurrentTime+NSExpire);
        
  return sql_execute("UPDATE nickserv "
   "SET username=%s, realhost=%s, publichost=%s, info=%s,"
   "t_ident=%d, t_seen=%d, t_expire=%s,"
   "status = %d WHERE snid=%d",
     sql_str(u->username), sql_str(u->realhost), sql_str(u->publichost), sql_str(u->info),
     irc_CurrentTime, irc_CurrentTime, sql_expire,
     u->status, snid);  
}

/* 
 * Checks a nick security settings
 *  - If the password matches (optional)
 *  - If the email is authenticated
 *  - If the password expired
 *  - If the authentication expired
 * Returns -1 if the password doesn't match
 */
int check_nick_security(u_int32_t snid, IRC_User *u, char* pass, char* email, int flags)
{
  char umodes[10];
  int i = 0;        /* umodes index */
  int diff = 1;
  int full_reg = 0; /* is nick fully registered ? */
  time_t t_lset_pass;
  time_t t_lset_answer;
  time_t t_lauth;
  MYSQL_RES *res;
  MYSQL_ROW row;
  
  res = sql_query("SELECT pass, t_lset_pass, t_lset_answer, t_lauth"
           " FROM nickserv_security WHERE snid=%d", snid);
           
  if(!res || !(row = sql_next_row(res)))
  {
    sql_free(res);
    log_log(ns_log, mod_info.name, "Missing nick security record for %d",
      snid);                  
    return -1;
  }
    
  if(pass)
  {
    if(row[0])
      diff = memcmp(hex_bin(row[0]), encrypted_password(pass), 16);  
    if(diff != 0)
    {
      sql_free(res);
      return -1;
    }
  }

  t_lset_pass = atoi(row[1]);
  t_lset_answer = atoi(row[2]);
  t_lauth = atoi(row[3]);

  if(DisableNickSecurityCode)
    full_reg = 1;
  else
  {
    if(IsNull(email))
      send_lang(u, nsu.u, MISSING_SET_EMAIL);
    else if(!(flags & NFL_AUTHENTIC))
      send_lang(u, nsu.u, MISSING_AUTH);
    else full_reg = 1;
  }
  
  if(NickPassExpire && (irc_CurrentTime - t_lset_pass > NickPassExpire))
  {
    send_lang(u, nsu.u, NICK_PASSWORD_EXPIRED);    
    full_reg = 0;
  }
  
  if(full_reg) /* nick is fully registered */
  {
    /* set the umodes on nick */
    umodes[i++] = '+';
    if(IsPrivateNick(u))
      umodes[i++] = 'p';
    umodes[i++] = 'r';
    umodes[i] = '\0';
    if(i > 1)
      irc_SvsMode(u, nsu.u, umodes);
    u->status |= NST_FULLREG;
  } else u->status &= ~NST_FULLREG;
  u->flags = flags;
  sql_free(res);
  return 0;
}
const int MVERSION = MODULES_VERSION;
