/******************************************************************
 * PTlink Services is (C) CopyRight PTlink IRC Software 1999-2005 *
 *                http://software.pt-link.net                     *
 * This program is distributed under GNU Public License           *
 * Please read the file COPYING for copyright information.        *
 ******************************************************************

  File: ns_identify.c
  Description: nickserv identify command

 *  $Id: ns_identify.c,v 1.1.1.1 2005/08/27 15:44:45 jpinto Exp $
*/

#include "stdinc.h"
#include "nickserv.h"
#include "ircservice.h"
#include "suser.h"
#include "dconf.h"
#include "s_log.h"
#include "log.h"
#include "lang.h"
#include "modules.h"
#include "ircsvs.h"
#include "encrypt.h"
#include "strhand.h"
#include "modevents.h"
#include "my_sql.h"
#include "lang/common.lh"
#include "lang/nscommon.lh"
#include "lang/nickserv.lh"
#include "lang/ns_identify.lh"

SVS_Module mod_info =
/* module, version, description */
{ "ns_identify", "2.0", "nickserv identify command" };

/* Change Log
  2.0 - 0000276: nick password expire option
        0000272: move nickserv security info to a specific table
        0000265: remove nickserv cache system  
  1.2 - Added missing code for the issue 0000245
      - 0000255: new field to store nick expire time
  1.1 - 0000245: use snid to track previous identify
*/
  
/* functions and events we require */
ServiceUser* (*nickserv_suser)(void);
int (*update_nick_online_info)(IRC_User* u, u_int32_t snid, int lang);
int (*check_nick_security)
  (u_int32_t snid, IRC_User *u, char* pass, char* email, int flags);
int e_nick_identify;                                         
int ns_log;

Module_Function mod_requires[] =
{
   MOD_FUNC(nickserv_suser)
   MOD_FUNC(e_nick_identify)
   MOD_FUNC(update_nick_online_info)
   MOD_FUNC(check_nick_security)
   { NULL }
};

/* functions and events we provide */


Module_Function mod_provides[] =
{
   { NULL }
};

/* internal functions */
int mod_load(char *param);
void mod_unload(void);
void ns_identify(IRC_User *s, IRC_User *u);

/* Local settings */
int FailedIdentifyAttempts;
int NSExpire = 30*24*60*60; /* lets put a default for safety */
int AgeBonusPeriod = 0;
int AgeBonusValue = 0;
                                                                              
/* Configuration load */
int get_local_dconf(void)
{
  DCONF_GET_VALUE(FailedIdentifyAttempts)
  DCONF_GET_VALUE(AgeBonusPeriod);
  DCONF_GET_VALUE(AgeBonusValue);
  DCONF_GET_VALUE(NSExpire);
  return 0;
}

/* Local variables */
ServiceUser* nsu;

int mod_load(char *param)
{

  ns_log = log_handle("nickserv");

  /* get service user */
  nsu = nickserv_suser();
          
  get_local_dconf();  
  suser_add_cmd(nsu, "IDENTIFY", 
    (void*) ns_identify, IDENTIFY_SUMMARY, IDENTIFY_HELP);
  	
  return 0;
}

void
mod_unload(void)
{
  suser_del_mod_cmds(nsu, &mod_info);
}
 
/* s = service the command was sent to
   u = user the command was sent from */
void ns_identify(IRC_User *s, IRC_User *u)
{
  u_int32_t snid;
  MYSQL_RES *res = NULL;
  MYSQL_ROW row;  
  char *pass = strtok(NULL, " ");
  char *extraopt = NULL;

  if(pass != NULL)
    extraopt = strtok(NULL, "");
    
  if(IsNull(pass))
    send_lang(u, s, IDENTIFY_SYNTAX);
  else if(u->snid)
    send_lang(u, s, ALREADY_IDENTIFIED);
  else if((res = sql_query("SELECT snid, flags, lang, email, ajoin"
    " FROM nickserv WHERE nick=%s", sql_str(irc_lower_nick(u->nick))))
      && (row = sql_next_row(res)))
  {
    u_int32_t flags = atoi(row[1]);    
    int lang = atoi(row[2]);
    char *email = row[3];
    char *ajoin = row[4];
    snid = atoi(row[0]);
    
    if(flags & NFL_FORBIDDEN)
    {
      send_lang(u,s, NICK_X_IS_FORBIDDEN, u->nick);
      sql_free(res);      
      return;
    }
    if(check_nick_security(snid, u, pass, email, flags) == -1)
    {
      log_log(ns_log, mod_info.name, "Nick %s failed identify by %s",
        u->nick, irc_UserSMask(u));
      if(FailedIdentifyAttempts && ++u->fcount>FailedIdentifyAttempts)
      {
        log_log(ns_log, mod_info.name, 
          "Killing %s on too many failed identify attempts", u->nick);
        irc_Kill(u, s, "Too many failed identify attempts");
      }
      else
        send_lang(u, s, INCORRECT_PASSWORD);
      sql_free(res);    
      return;
    }
    else
    {
      send_lang(u, s, IDENTIFY_OK); 
      log_log(ns_log, mod_info.name, "Nick %s identified by %s",
        u->nick, irc_UserSMask(u));    
      update_nick_online_info(u, snid, lang);      
      if(ajoin && (!extraopt || strcasecmp(extraopt,"NOJOIN") != 0))      
	irc_SvsJoin(u, s, ajoin);
	
      irc_CancelUserTimerEvents(u); /* delete the pending change nick event */
      mod_do_event(e_nick_identify, u, &snid);
    }
  }
  else
    send_lang(u, s, NICK_NOT_REGISTERED);    
  sql_free(res);
}

const int MVERSION = MODULES_VERSION;
