/**********************************************************************
 * PTlink IRC Services is (C) CopyRight PTlink IRC Software 1999-2005 *
 *                     http://software.pt-link.net                    *
 * This program is distributed under GNU Public License               *
 * Please read the file COPYING for copyright information.            *
 **********************************************************************  

  Description: nickserv info command
                                                                                
 *  $Id: ns_info.c,v 1.1.1.1 2005/08/27 15:44:43 jpinto Exp $
*/
#include "stdinc.h"
#include "strhand.h"
#include "nickserv.h"
#include "ircservice.h"
#include "suser.h"
#include "dconf.h"
#include "s_log.h"
#include "lang.h"
#include "modules.h"
#include "ircsvs.h"
#include "ns_group.h"  /* is_soper */
#include "modevents.h"
#include "my_sql.h"
#include "lang/common.lh"
#include "lang/nscommon.lh"
#include "lang/nickserv.lh"
#include "lang/ns_info.lh"

SVS_Module mod_info =
/* module, version, description */
{"ns_info", "2.1", "nickserv info command" };

/* Change Log
  2.1 - 0000306: nickserv info shows nick as private everytime it is online
      - 0000304: nickserv last seen only on info when nick is offline
      - 0000298: nickserv info now displays userhost
      - 0000303: expire time not displayed if nick has noexpire flag
  2.0 - 0000274: empty options field on ns info for authenticated nicks
      - 0000265: remove nickserv cache system
      - when no target is given assume is "self" info
  1.1 - 0000256: ago time split in larger time units
*/
  
/* external functions we need */
ServiceUser* (*nickserv_suser)(void);
int e_nick_info;

Module_Function mod_requires[] =
{
  MOD_FUNC(e_nick_info)
  MOD_FUNC(nickserv_suser)
  MOD_FUNC(is_soper)
  { NULL }
};

/* available commands from module */
void ns_info(IRC_User *s, IRC_User *u);

/* Local settings */
int AgeBonusPeriod = 0;
int AgeBonusValue = 0;
int NSExpire = 0;

/* internal functions */
int mod_load(char *param);
void mod_unload(void);
char* ago_time(time_t t, IRC_User *u);

/* Configuration load */
int get_local_dconf(void)
{
  DCONF_GET_VALUE(AgeBonusPeriod)
  DCONF_GET_VALUE(AgeBonusValue)
  DCONF_GET_VALUE(NSExpire)
  return 0;
}

/* Local variables */
ServiceUser* nsu;

int mod_load(char *param)
{
  nsu = nickserv_suser();  
  get_local_dconf();
  suser_add_cmd(nsu, "INFO", (void*) ns_info, INFO_SUMMARY, INFO_HELP);  
  return 0;
}

void
mod_unload(void)
{
  /* delete all our commands*/
  suser_del_mod_cmds(nsu, &mod_info);
}

char* ago_time(time_t t, IRC_User *u) 
{
  int years,months,days,hours,minutes,seconds;    

  t = irc_CurrentTime -t;
  years = t/(12*30*24*3600);
  t %= 12*30*24*3600;
  months = t/(30*24*3600);
  t %= 30*24*3600;
  days = t/(24*3600);
  t %= 24*3600;
  hours = t/3600;
  t %= 3600;
  minutes = t/60;
  t %= 60;
  seconds = t;
  if(years)
    return lang_str(u, AGO_TIME_Y,
      years,
      months,
      days);
  else
  if(months)
    return lang_str(u, AGO_TIME_M,
      months,
      days,
      hours);
  if(days)
    return lang_str(u, AGO_TIME_D,
      days,
      hours,
      minutes);
  else
    return lang_str(u,AGO_TIME,
      hours,
      minutes,
      seconds);                                                                                
}
 
/* s = service the command was sent to
   u = user the command was sent from */
void ns_info(IRC_User *s, IRC_User *u)
{
  char buf[64];
  struct tm *tm;
  char *target; 
  char *langstr = NULL;  
  int sadmin = 0;
  u_int32_t source_snid = u->snid;    
  
  target = strtok(NULL, " ");
  
  /* assume "self" info */
  if(IsNull(target) && u->snid)
    target = u->nick;
  
  if(IsNull(target))
    send_lang(u, s, NICK_INFO_SYNTAX);
  else if(sql_singlequery("SELECT "
    "snid," 		/* 0 */
    "flags,"		/* 1 */
    "status,"		/* 2 */
    "t_reg,"		/* 3 */
    "t_ident,"		/* 4 */
    "t_seen,"		/* 5 */
    "t_expire,"		/* 6 */
    "lang,"		/* 7 */
    "email,"		/* 8 */
    "ajoin,"		/* 9 */
    "realhost,"         /* 10 */
    "publichost,"       /* 11 */   
    "username"          /* 12 */
    " FROM nickserv WHERE nick=%s", sql_str(irc_lower_nick(target))
      ) == 0)
    send_lang(u, s, NICK_X_NOT_REGISTERED, target);
  else /* everything is valid lets check the auth */
    {
      int c = 0;
      u_int32_t snid = sql_field_i(c++);
      u_int32_t flags = sql_field_i(c++);
      u_int32_t status = sql_field_i(c++);
      time_t t_reg  = sql_field_i(c++);
      time_t t_ident  = sql_field_i(c++);
      time_t t_seen = sql_field_i(c++);
      time_t t_expire = sql_field_i(c++);
      int lang = sql_field_i(c++);
      char *email = sql_field(c++);
      char *ajoin = sql_field(c++);
      char *realhost = sql_field(c++);
      char *publichost = sql_field(c++);
      char *username = sql_field(c++);
      
      if(flags & NFL_FORBIDDEN)
      {
        send_lang(u,s, NICK_X_IS_FORBIDDEN, target);
        return;
      }
      sadmin = (is_soper(source_snid) != 0);
      send_lang(u, s, NICK_INFO_HEADER);
      send_lang(u, s, NICK_NICK_X_X, target, 
        (status & NST_ONLINE) ? "(ONLINE)" : "");
      if((flags & NFL_PRIVATE) && (source_snid != snid) && !sadmin)
        send_lang(u, s, NICK_INFO_PRIVATE, target);
      else
        {
          if(sadmin)
            send_lang(u, s, NICK_SNID, snid);
	 
	 /* print userhost */
         snprintf(buf, sizeof(buf), "[%s]", realhost);    
         send_lang(u, s, NICK_USERHOST, username, publichost, 
		(sadmin || (source_snid == snid)) ? buf : "");
	 
	  tm = localtime(&t_reg);
          strftime(buf, sizeof(buf), format_str(u, DATE_FORMAT), tm);
          send_lang(u, s, NICK_REGDATE_X_X, buf, ago_time(t_reg, u));
          tm = localtime(&t_ident);
          strftime(buf, sizeof(buf), format_str(u, DATE_FORMAT), tm);
          send_lang(u, s, NICK_IDDATE_X_X, buf, ago_time(t_ident, u));          
          if(!(status & NST_ONLINE))
	  {
	    tm = localtime(&t_seen);
            strftime(buf, sizeof(buf), format_str(u, DATE_FORMAT), tm);
            send_lang(u, s, NICK_SEENDATE_X_X, buf, ago_time(t_seen, u));          
          }
	  if(!(flags & NFL_NOEXPIRE))
	  {
	    tm = localtime(&t_expire);
            strftime(buf, sizeof(buf), format_str(u, DATE_FORMAT), tm);
            send_lang(u, s, NICK_EXPIRES_X, buf);                    
          }
	  index2lang(lang, langstr);
          if(langstr)
            send_lang(u, s, NICK_LANGUAGE, langstr);
          if(sadmin || !(flags & NFL_HIDEEMAIL) || (u->snid == snid))
            {
              if(!IsNull(email))
                {
                  if(flags & NFL_AUTHENTIC)
                    send_lang(u, s, NICK_EMAIL, email);
                  else
                    send_lang(u, s, NICK_EMAIL_NOAUTH, email);
                }
            }
#if 0 
  /* This will be moved to the nick profile */           
          if(!IsNull(nr->url))
            send_lang(u, s, NICK_URL, nr->url);
          if(!IsNull(nr->location))
            send_lang(u, s, NICK_LOCATION, nr->location);
          if(!IsNull(nr->imid))
            send_lang(u, s, NICK_IMID, nr->imid);            
#endif            
          if(!IsNull(ajoin))
            send_lang(u, s, NICK_AJOIN, ajoin);
          if((flags &~ NFL_AUTHENTIC) && (sadmin || (source_snid == snid)))
            send_lang(u, s, NICK_OPTIONS_X, 
              mask_string(nick_options_mask, flags));
        }
      mod_do_event(e_nick_info, u, &snid);
      send_lang(u, s, NICK_INFO_TAIL);        
    }
}

const int MVERSION = MODULES_VERSION;
