/******************************************************************
 * PTlink Services is (C) CopyRight PTlink IRC Software 1999-2005 *
 *                http://software.pt-link.net                     *
 * This program is distributed under GNU Public License           *
 * Please read the file COPYING for copyright information.        *
 ******************************************************************
                                                                                
  File: ns_list.c
  Description: nickserv list command
                                                                                
 *  $Id: ns_list.c,v 1.1.1.1 2005/08/27 15:44:44 jpinto Exp $
*/
#include "stdinc.h"
#include "ircservice.h"
#include "suser.h"
#include "dconf.h"
#include "s_log.h"
#include "log.h"
#include "lang.h"
#include "modules.h"
#include "strhand.h"
#include "modevents.h"
#include "ircsvs.h"
#include "encrypt.h"
#include "nickserv.h"
#include "nsmacros.h"
#include "lang/common.lh"
#include "lang/nscommon.lh"
#include "lang/ns_list.lh"
#include "my_sql.h"
#include "ns_group.h"

SVS_Module mod_info =
/* module, version, description */
{ "ns_list", "2.0","nickserv list command" };

/* Change Log
  2.0 - 0000279: nickserv list support for flags (auth, noexpire, forbidden ...)
      - 0000265: remove nickserv cache system
  1.2 - we don't need irc_lower()
  1.1 - 0000246: help display with group filter
  */
  
/* external functions we need */
ServiceUser* (*nickserv_suser)(void);
u_int32_t (*find_group)(char *name);
 
Module_Function mod_requires[] =
{
   MOD_FUNC(nickserv_suser)
   MOD_FUNC(is_sadmin)
   MOD_FUNC(find_group)
   { NULL }
};

/* internal functions */
int mod_load(char *param);
void mod_unload(void);

/* available commands from module */
void ns_list(IRC_User *s, IRC_User *u);

/* Local settings */

/* Configuration load */
int get_local_dconf(void)
{
  return 0;
}

/* Local vars */
ServiceUser* nsu;
int ns_log;

int mod_load(char *param)
{
  ns_log = log_handle("nickserv");
  nsu = nickserv_suser();
  get_local_dconf();
  suser_add_cmd_g(nsu, "LIST", ns_list, LIST_SUMMARY, LIST_HELP,
    find_group("Admin"));
  return 0;
}

void mod_unload(void)
{
  suser_del_mod_cmds(nsu, &mod_info);     
}


/* s = service the command was sent to
   u = user the command was sent from */
void ns_list(IRC_User *s, IRC_User *u)
{
  u_int32_t source_snid;
  char* mask;
  char *options;
  int count = 0;
  char sql[1024];
  char buf[128];
  
  mask = strtok(NULL, " ");
  options = strtok(NULL, " ");
    
  CHECK_IF_IDENTIFIED_NICK
    
  if(!is_sadmin(source_snid))
    send_lang(u, s, ONLY_FOR_SADMINS);
  else if(IsNull(mask))
    send_lang(u, s, LIST_SYNTAX);
  else
  {
    MYSQL_RES* res;
    MYSQL_ROW row;
    char *crit;
    char *c;
    u_int32_t flags = 0;
    
    if(mask[0] == '@')
    {      
      crit = "email";
      mask++;
    }
    else if(mask[0] == '#')
    {      
      crit = "realhost";
      mask++;
    }        
    else if(mask[0] == '!')
    {
      crit = "concat(username,'@',publichost)";
      mask++;
    }
    else
    {
      crit = "nick";
    }        
        
    /* replace '*' with '%' for sql */
    while((c = strchr(mask,'*'))) *c='%';
    snprintf(sql, sizeof(sql), "SELECT nick, email, username, publichost, realhost"
      " FROM nickserv WHERE %s LIKE %s", crit, sql_str(mask));

    if(options)
      {
        if(strstr(options, "auth"))
          flags |= NFL_AUTHENTIC;
        if(strstr(options, "noexpire"))
          flags |= NFL_NOEXPIRE;
        if(strstr(options, "forbidden"))
          flags |= NFL_FORBIDDEN;
        if(strstr(options, "nonews"))
          flags |= NFL_NONEWS;                  
      }
    if(flags)
    {
      snprintf(buf, sizeof(buf), " AND (flags & %d) ", flags);
      strcat(sql, buf);
    }
    strcat(sql, "ORDER BY nick");
      
    res = sql_query("%s", sql);
    if(res)
      send_lang(u, s, NICK_LIST_HEADER_X,
        mysql_num_rows(res));
    while((row = sql_next_row(res)))
    {
      snprintf(buf, sizeof(buf), "%s@%s", 
        row[2] ? row[2] : "", row[3] ? row[3] : "");
      send_lang(u, s, NICK_LIST_X_X_X_X, 
        row[0] , row[1] ? row[1]: "", buf, row[4] ? row[4] : "");
      if(++count >= 50) /* avoid flood */
      {
        send_lang(u, s, LIST_STOPPED);
          break;
      }
    }
    send_lang(u, s, NICK_LIST_TAIL);
    sql_free(res);
  }
}

const int MVERSION = MODULES_VERSION;
