/******************************************************************
 * PTlink Services is (C) Copyright PTlink IRC Software 1999-2005 *
 *                http://software.pt-link.net                     *
 * This program is distributed under GNU Public License           *
 * Please read the file COPYING for copyright information.        *
 ******************************************************************
                                                                                
  File: ns_login.c
  Description: nickserv login command
                                                                                
 *  $Id: ns_login.c,v 1.1.1.1 2005/08/27 15:44:30 jpinto Exp $
*/
#include "stdinc.h"
#include "strhand.h"
#include "nickserv.h"
#include "ircservice.h"
#include "suser.h"
#include "dconf.h"
#include "s_log.h"
#include "log.h"
#include "lang.h"
#include "modules.h"
#include "ircsvs.h"
#include "encrypt.h"
#include "modevents.h"
#include "my_sql.h"
#include "nsmacros.h"
#include "lang/common.lh"
#include "lang/nscommon.lh"
#include "lang/nickserv.lh"
#include "lang/ns_login.lh"

SVS_Module mod_info =
/* module, version, description */
{ "ns_login", "2.0","nickserv login command" };
/* Change Log
  2.0 - 0000287: ns_getpass and ns_getsec to recover password and security code
        0000272: move nickserv security info to a specific table
        0000265: remove nickserv cache system
  1.2 - 0000248: nick login log message using wrong string pointer
      - 0000249: add nickserv REGAIN alias to LOGIN
  1.1 - call the nick_identify event on login
*/
          

/* external functions we need */
ServiceUser* (*nickserv_suser)(void);
int e_nick_identify;
 
Module_Function mod_requires[] =
{
  MOD_FUNC(nickserv_suser)
  MOD_FUNC(e_nick_identify)
  { NULL }
};

/* internal functions */
int mod_load(char *param);
void mod_unload(void);

/* available commands from module */
void ns_login(IRC_User *s, IRC_User *u);
void ns_recover(IRC_User *s, IRC_User *u);

/* Local settings */
int FailedIdentifyAttempts;

/* Configuration load */
int get_local_dconf(void)
{
  DCONF_GET_VALUE(FailedIdentifyAttempts)  
  return 0;
}

ServiceUser *nsu;
int ns_log;

static int is_recover = 0; /* internal flag for the recover command */
int mod_load(char *param)
{
  get_local_dconf();
  nsu = nickserv_suser();
  ns_log = log_handle("nickserv");  
  suser_add_cmd(nsu, "LOGIN", ns_login, LOGIN_SUMMARY, LOGIN_HELP);
  suser_add_cmd(nsu, "GHOST", ns_login, NULL, NULL);
  suser_add_cmd(nsu, "REGAIN", ns_login, NULL, NULL);
  suser_add_cmd(nsu, "RECOVER", ns_recover, NULL, NULL);
  return 0;
}

void mod_unload(void)
{
  suser_del_mod_cmds(nsu, &mod_info);   
}
 
/* s = service the command was sent to
   u = user the command was sent from */
void ns_login(IRC_User *s, IRC_User *u)
{
  int diff;
  
  char *target = strtok(NULL, " ");
  char *pass = strtok(NULL, " ");
        
  if(IsNull(target) || IsNull(pass)) /* all parts filled in ? */
    send_lang(u, s, NICK_LOGIN_SYNTAX);
  else if(irccmp(target, u->nick) == 0) /* self login */
    send_lang(u, s, NO_SELF_LOGIN);
  else if(sql_singlequery("SELECT snid, flags, lang"
    " FROM nickserv WHERE nick=%s", sql_str(irc_lower_nick(target))) == 0)
    send_lang(u, s, NICK_NOT_REGISTERED);
  else  /* ok to validate password */
  {
    char *check;  
    u_int32_t snid = sql_field_i(0);
    u_int32_t flags = sql_field_i(1);
    int lang = sql_field_i(2);


    if(flags & NFL_FORBIDDEN)
    {
      send_lang(u,s, NICK_X_IS_FORBIDDEN, target);
      return;
    }
    
    check = is_recover ? "securitycode" : "pass";
    if(sql_singlequery("SELECT %s"
      " FROM nickserv_security WHERE snid=%d", check, snid) == 0)
    {
      send_lang(u, s, INCORRECT_PASSWORD);
      log_log(ns_log, mod_info.name, "Missing nick security record for %d",
        snid);
    }
      
    if(sql_field(0))
    {
      if(is_recover)
      { /* case insentive for md5 validation */
        diff = strcasecmp(sql_field(0), pass);
        is_recover = 0;
      }
      else
      {
        diff = memcmp( hex_bin(sql_field(0)), encrypted_password(pass), 16);
      }
    }
      
    if(diff !=0 )
    {
      log_log(ns_log, mod_info.name, "Failed login for %s by %s",
        target, irc_UserMask(u));            
      if(FailedIdentifyAttempts && ++u->fcount>FailedIdentifyAttempts)
      {
        log_log(ns_log, mod_info.name, 
          "Killing %s after too many failed identifies", u->nick);
        irc_Kill(u, s, "Too many invalid identify attempts");
      }
      else
        send_lang(u, s, INCORRECT_PASSWORD);
    }
    else
    {
      IRC_User* ku; /* we may beed to kill the current used */
      u->lang = lang;
      ku = irc_FindUser(target);
      log_log(ns_log, mod_info.name, "Nick %s login by %s",
        target, irc_UserMask(u));
      u->req_snid = snid; /* for auto-identify */
      /* kill the user */
      if(ku)
      {
        char killmsg[128];
        snprintf(killmsg, sizeof(killmsg), 
          "LOGIN command used by %s", u->nick);
        irc_Kill(ku, s, killmsg);
      }
      irc_SvsNick(u, s, target);
    }
  }
}

/* s = service the command was sent to
   u = user the command was sent from */
void ns_recover(IRC_User *s, IRC_User *u)
{
  is_recover = 1;
  ns_login(s, u);
  return;
}

const int MVERSION = MODULES_VERSION;
