/**********************************************************************
 * PTlink IRC Services is (C) CopyRight PTlink IRC Software 1999-2005 *
 *                     http://software.pt-link.net                    *
 * This program is distributed under GNU Public License               *
 * Please read the file COPYING for copyright information.            *
 **********************************************************************  

  Description: nickserv register command

 *  $Id: ns_register.c,v 1.1.1.1 2005/08/27 15:44:27 jpinto Exp $
*/

#include "nickserv.h"
#include "stdinc.h"
#include "ircservice.h"
#include "suser.h"
#include "dconf.h"
#include "s_log.h"
#include "log.h"
#include "lang.h"
#include "modules.h"
#include "strhand.h"
#include "ircsvs.h"
#include "encrypt.h"
#include "my_sql.h"
#include "sqlb.h"
#include "email.h"
#include "lang/nscommon.lh"
#include "lang/ns_register.lh"

SVS_Module mod_info =
/* module, version, description */
{"ns_register", "2.0","nickserv register command" };

/* Change Log
  2.0 - 0000277: ns_blacklist to support email blacklists
        0000265: remove nickserv cache system
  1.2 - 0000255: new field to store nick expire time
  1.1 - 0000218: NickDefaultOptions to set nick default options
*/
            
/* functions/events we require */
ServiceUser* (*nickserv_suser)(void);
int (*forbidden_email)(char *email);
int mysql_connection;
int ns_log;

Module_Function mod_requires[] =
{

  MOD_FUNC(nickserv_suser)
  MOD_FUNC(mysql_connection)  
  { NULL }
};

Module_Function mod_options[] =
{

  MOD_FUNC(forbidden_email)
  { NULL }
};

/* functions and events we provide */
int e_nick_register;                                                                            
Module_Function mod_provides[] =
{
   MOD_FUNC(e_nick_register)
   { NULL }
};

/* internal functions */
int mod_load(char *param);
void mod_unload(void);
void ns_register(IRC_User *s, IRC_User *u);
void parse_nick_def_options(void);

/* Local settings */
int DisableNickSecurityCode = 0;
int ForceStrongPasswords = 0;
char *EmailFrom = NULL;
char *EmailFromName = NULL;
int MaxNicksPerMail = 0;
int SecurityCodeLenght = 0;
char *NickProtectionPrefix = NULL;
char *NickDefaultOptions = NULL;
int NSExpire = 0;

/* Configuration load */
int get_local_dconf(void)
{
  DCONF_GET_VALUE(ForceStrongPasswords)
  DCONF_GET_STRVALUE(EmailFrom)
  DCONF_GET_STRVALUE(EmailFromName)  
  DCONF_GET_VALUE(MaxNicksPerMail)
  DCONF_GET_VALUE(DisableNickSecurityCode)
  DCONF_GET_VALUE(SecurityCodeLenght)
  DCONF_GET_VALUE(NSExpire);
  DCONF_GET_STRVALUE(NickProtectionPrefix)
  DCONF_GET_STRVALUE(NickDefaultOptions)
  return 0;
}

static ServiceUser* nsu;
u_int32_t nick_def_options = 0;
char* welcome_emails[MAX_LANGS];

int mod_load(char *param)
{

  ns_log = log_handle("nickserv");
  
  if(email_load("welcome", welcome_emails) < 0)
    return -1;      
    
  get_local_dconf();
  parse_nick_def_options();

  nsu = nickserv_suser();
  suser_add_cmd(nsu,
    "REGISTER", (void*) ns_register, REGISTER_SUMMARY, REGISTER_HELP);
  return 0;
}

void
mod_unload(void)
{
   suser_del_mod_cmds(nsu, &mod_info);
   email_free(welcome_emails);
}
 
/* s = service the command was sent to
   u = user the command was sent from */
void ns_register(IRC_User *s, IRC_User *u)
{
  
  char *pass = strtok(NULL, " ");    
  char *email = strtok(NULL, " ");
  
  /* syntax validation */
  if((IsNull(pass)) || (IsNull(email)))
    send_lang(u, s, NICK_REGISTER_SYNTAX);    
  /* check requirements */
  else if(!is_email(email)) 
    send_lang(u, s, INVALID_EMAIL);
  else if(ForceStrongPasswords && is_weak_passwd(pass))
    send_lang(u, s, WEAK_PASSWORD);
  else if(NickProtectionPrefix &&
    !strncasecmp(u->nick, NickProtectionPrefix, strlen(NickProtectionPrefix)))
      send_lang(u, s, NICK_CANNOT_BE_REGISTERED, u->nick);
  else if(nick2snid(u->nick) != 0)
    send_lang(u, s, NICK_ALREADY_REGISTERED);
  else if(forbidden_email && (forbidden_email(email) > 0))
    send_lang(u, s, FORBIDDEN_EMAIL);
  else if(MaxNicksPerMail && (reg_count_for_email(email) >= MaxNicksPerMail))
    send_lang(u, s, ALREADY_X_WITH_EMAIL, MaxNicksPerMail);
  else /* execute operation */
    {
      u_int32_t snid = 0;
      char* securitycode;
      sqlb_init("nickserv");
      sqlb_add_str("nick", irc_lower_nick(u->nick));
      sqlb_add_str("email", email);
      sqlb_add_int("flags", nick_def_options);
      sqlb_add_int("status", NST_ONLINE);
      sqlb_add_int("t_reg", irc_CurrentTime);
      sqlb_add_int("t_ident", irc_CurrentTime);
      sqlb_add_int("t_seen", irc_CurrentTime);      
      sqlb_add_int("t_expire", irc_CurrentTime + NSExpire);            
      sqlb_add_str("username", u->username);
      sqlb_add_str("realhost", u->realhost);
      sqlb_add_str("publichost", u->publichost);
      sqlb_add_str("info", u->info);

      sqlb_add_int("lang", u->lang);
      securitycode = malloc(SecurityCodeLenght+1);
      rand_string(securitycode, SecurityCodeLenght, SecurityCodeLenght);

      snid = sql_execute("%s", sqlb_insert());
      if(snid == 0)
      {
        free(securitycode);
        send_lang(u, s, NICK_REGISTER_FAIL);
        return ;
      } 
      else
      {
        sqlb_init("nickserv_security");              
        sqlb_add_int("snid", snid);
        sqlb_add_str("pass", hex_str(encrypted_password(pass),16));
        sqlb_add_str("securitycode", 
          hex_str(encrypted_password(securitycode),16));
        sqlb_add_int("t_lset_pass", (int) irc_CurrentTime);
        sql_execute("%s", sqlb_insert());
      }
      u->snid = snid;
      u->flags = nick_def_options;
      u->status = NST_ONLINE;      
      if(DisableNickSecurityCode == 0)
        {

          email_init_symbols();
          email_add_symbol("from", EmailFrom);
          email_add_symbol("from_name", EmailFromName);
          email_add_symbol("nick", u->nick);
          email_add_symbol("email", email);
          email_add_symbol("securitycode", securitycode);
          if(email_send(welcome_emails[u->lang]) < 0)
            {
              log_log(ns_log, mod_info.name, "Error sending welcome email to %s by %s",
                email, irc_UserMask(u));
              send_lang(u, s, WELCOME_ERROR);
            }
          else
            {
              log_log(ns_log, mod_info.name, "Welcome email was sent to %s by %s",
                email, irc_UserMask(u));
              send_lang(u, s, WELCOME_SENT);
            }  

        }
      else /* no security code required */
        {
          irc_SvsMode(u, s, "+r");        
          send_lang(u, s, NICK_REGISTER_SUCCESS);
        }
      free(securitycode);
      log_log(ns_log, mod_info.name, "Nick %s [%s] registered by %s",
        u->nick, email, irc_UserSMask(u));        
    }
}
/* parse NickDefaultOptions */
void parse_nick_def_options(void)
{
  char* options;
  char* opt;
  
  if(NickDefaultOptions == NULL)
    return;
    
  options = strdup(NickDefaultOptions);  
  opt = strtok(options,",");
  while(opt)
    {
      int i;
      i = mask_value(nick_options_mask, opt);
      if( i == 0)
      	errlog(L_ERROR,"Unknown NickDefaultOption %s", opt);
      else
      	nick_def_options |= i;
      opt = strtok(NULL,",");
    }
  FREE(options);
}

const int MVERSION = MODULES_VERSION;
