/**********************************************************************
 * PTlink IRC Services is (C) CopyRight PTlink IRC Software 1999-2005 *
 *                     http://software.pt-link.net                    *
 * This program is distributed under GNU Public License               *
 * Please read the file COPYING for copyright information.            *
 **********************************************************************  

  Description: operserv module

 *  $Id: operserv.c,v 1.1.1.1 2005/08/27 15:44:35 jpinto Exp $
*/

#include "stdinc.h"
#include "ircsvs.h"
#include "ircservice.h"
#include "ns_group.h" /* we need is_oper/admin() */
#include "suser.h"
#include "dconf.h"
#include "modules.h"
#include "path.h"
#include "lang.h"
#include "strhand.h"
#include "log.h"
#include "s_log.h"
#include "lang/common.lh"
#include "lang/operserv.lh"

/* module, version, description */
SVS_Module mod_info =
{"operserv", "2.0", "operserv core module" };

/* Change Log
  2.0 - 0000273: +a only set on sadmins if OperChan is defined
        0000265: remove nickserv cache system
*/

/* functions we require */
Module_Function mod_requires[] =
{
  MOD_FUNC(is_soper)
  MOD_FUNC(is_sadmin)
  { NULL }
};

/* functions we provide */
ServiceUser* operserv_suser(void);                                                                                
Module_Function mod_provides[] = {
  MOD_FUNC(operserv_suser)
  { NULL }
};

char* OS_Nick;
char* OS_Username;
char* OS_Host;
char* OS_Info;
char* OSLogChan;
char* OperChan;
char* SAdminChan;
int OperControl;

ConfItem mod_dconf[] = {
	DCONF_DESC("OperServ core configuration")
	DCONF_STRING(OS_Nick, CF_REQ)
	DCONF_STRING(OS_Username, CF_REQ)
	DCONF_STRING(OS_Host, CF_REQ)
	DCONF_STRING(OS_Info, CF_REQ)
	DCONF_STRING(OSLogChan, 0)
	DCONF_STRING(OperChan, 0)
        DCONF_STRING(SAdminChan, 0)	
	DCONF_SET(OperControl, 0)
	{ NULL }
};

ServiceUser osu;
int os_log;

/* internal functions */
int mod_load(char *param);
void mod_unload(void);
void ev_os_oper(IRC_User *u);

/* core events */
void ev_os_new_user(IRC_User* u, void *s);

/* commands */
void os_unknown(IRC_User* s, IRC_User* t);

int mod_load(char *param)
{
  /* open a log file */
  os_log = log_open("operserv","operserv");
  
  if(os_log<0)
  {
    errlog(L_ERROR, "Could not open operserv log file!");
    return -1;
  }

  /* Create operserv user */      
  osu.u = irc_CreateLocalUser(OS_Nick, OS_Username, OS_Host, OS_Host,
    OS_Info,"+ro");
  
  if(OSLogChan)
  {
    IRC_Chan *chan;
    log_set_irc(os_log, OS_Nick, OSLogChan);
    chan = irc_ChanJoin(osu.u, OSLogChan, CU_MODE_ADMIN|CU_MODE_OP);
    irc_ChanMode(osu.u, chan, "+Ostn");
  }
          
  /* Add msg events for child modules */
  irc_AddUMsgEvent(osu.u, "*", (void*) os_unknown); /* any other msg handler */
    
  /* Add user events */
  
  /* New user for hostrule functions */
  irc_AddEvent(ET_NEW_USER, (void*) ev_os_new_user); /* new user */
  
  /* Mode change for "on oper" functions */
  irc_AddUmodeChange("+o", ev_os_oper);
  
  return 0;
}

void
mod_unload(void)
{

  /* remove operserv and all associated events */
  irc_QuitLocalUser(osu.u, "Removing service");

  /* remove irc events */
  irc_DelEvent(ET_NEW_USER, (void*) ev_os_new_user);
}
 
void os_unknown(IRC_User* s, IRC_User* t)
{
  send_lang(t, s, UNKNOWN_COMMAND, irc_GetLastMsgCmd());
}


void ev_os_new_user(IRC_User* u, void *s)
{
 /* maybe we will need something here */
}

void ev_os_oper(IRC_User *u)
{  
  if(OperControl && (is_soper(u->snid) == 0))
  {
    send_lang(u, osu.u, NOT_REGISTERED_OPER);
    irc_SvsMode(u, osu.u, "-o");
    return;
  }
  if(OperChan)
  {
    irc_CNameInvite(OperChan, u, osu.u);
    irc_SvsJoin(u, osu.u, OperChan);
  }
  if(is_sadmin(u->snid) != 0) /* is an sadmin */
  {
    irc_SvsMode(u, osu.u, "+a");
    if(SAdminChan)
    {
      irc_CNameInvite(SAdminChan, u, osu.u);
      irc_SvsJoin(u, osu.u, SAdminChan);            
    }            
  }  
}

/* to return the operserv client */
ServiceUser* operserv_suser(void)
{
  return &osu;
}

const int MVERSION = MODULES_VERSION;
