/**********************************************************************
 * PTlink IRC Services is (C) CopyRight PTlink IRC Software 1999-2005 *
 *                     http://software.pt-link.net                    *
 * This program is distributed under GNU Public License               *
 * Please read the file COPYING for copyright information.            *
 **********************************************************************  

  Description: operserv hostrule
                                                                                
 *  $Id: os_hostrule.c,v 1.2 2005/09/07 18:27:35 jpinto Exp $
*/
#include "stdinc.h"
#include "ircservice.h"
#include "path.h"
#include "suser.h"
#include "dconf.h"
#include "strhand.h" /* for time_str */
#include "lang.h"
#include "modules.h"
#include "ircsvs.h"
#include "my_sql.h"
#include "s_log.h"
#include "log.h"
#include "os_hostrule.h"
#include "modevents.h"
#include "lang/common.lh"	/* wee need DATE_FORMAT */
#include "lang/os_hostrule.lh"

SVS_Module mod_info =
/* module, version, description */
{"os_hostrule", "1.5", "operserv hostrule command" };
/* Change Log
  1.5 - 0000323: os_hostrule not working with some rule types
  1.4 - 0000309: HOSTULE ADD syntax changed to ADD TYPE HOST [+param] Message
        0000300: aglines not properly set
  1.3 - 0000294: bot hostrule parameter incorrectly converted to time
  1.2 - 0000262: os_hostrule crashing on module unload
        0000253: hostrule not being deleted after expire
  1.1 - The gline dconf option was fixed to a time setting
        We now pass the gline author as expected
*/

#define DB_VERSION	1

/* external functions we need */
ServiceUser* (*operserv_suser)(void);
int *mysql_connection;
int e_expire;

Module_Function mod_requires[] =
{
  MOD_FUNC(e_expire) /* we need this to run the expire routines */
  MOD_FUNC(mysql_connection)
  MOD_FUNC(operserv_suser)
  { NULL }
};

/* dconf settings we use */
int DefaultHRTime;
char* HR_Nick;
char* HR_Username;
char* HR_Host;
char* HR_Info;
int DefSessionLimit = 0;
int MaxSessionHits;
int SessionsGlineTime;

ConfItem mod_dconf[] = {
  DCONF_DESC("HostRules configuration")
  DCONF_STRING(HR_Nick, CF_REQ)
  DCONF_STRING(HR_Username, CF_REQ)
  DCONF_STRING(HR_Host, CF_REQ)
  DCONF_STRING(HR_Info, CF_REQ)
  DCONF_TIME(DefaultHRTime, CF_REQ)	
  DCONF_INT(DefSessionLimit, 0)
  DCONF_INT(MaxSessionHits, 0)
  DCONF_TIME(SessionsGlineTime, 0)	
  { NULL }
};

/* internal functions */
int mod_load(char *param);
void mod_unload(void);
void os_hostrule(IRC_User *s, IRC_User *u);
void ev_os_hostrule_new_user(IRC_User* u, void *s);
void ev_os_hostrule_quit(IRC_User* u, char *lastquit);
void ev_os_hostrule_kill(IRC_User* u, IRC_User *killer);
int string2rtt(char *str); /* returns index on the ruletype table */
HostRecord* add_to_session(char *host);
void del_from_session(char *host);
HostRecord* find_session(char *host);
unsigned int hash_host(const char* host);
int ev_os_hostrule_expire(void* dummy1, void* dummy2);

/* available commands from module */
int hr_message(IRC_User* user, HostRule* rule);
int hr_notice(IRC_User* user, HostRule* rule);
int hr_agline(IRC_User* user, HostRule* rule);
int hr_bot(IRC_User* user, HostRule* rule);
int hr_zombie(IRC_User* user, HostRule* rule);

/* host rule types 
 NOTE: rules >511 are never expiring rules
*/
#define		HR_MESSAGE  1
#define		HR_NOTICE   2
#define		HR_AGLINE   3
#define		HR_ZOMBIE		4
#define		HR_BOT			512


/* Local variables */
HostRuleType ruletypes[]= {
  { "MESSAGE", HR_MESSAGE, hr_message },
  { "NOTICE", HR_NOTICE, hr_notice },
  { "AGLINE", HR_AGLINE, hr_agline },
  { "ZOMBIE", HR_ZOMBIE, hr_zombie },
  { "BOT", HR_BOT, hr_bot },
  { NULL }
};

ServiceUser hrsu;	/* HostRule user */    
ServiceUser* osu;	/* OperServ user */
int os_log = 0;  

#define HOST_HSIZE 16384

/* internal structures/functions */
HostRecord *sessionTable[HOST_HSIZE];

static HostRule* RulesList = NULL;

int mod_load(char *param)
{
  if(sql_check_inst_upgrade(mod_info.name, DB_VERSION, NULL) < 0)
    return -4;

  os_log = log_handle("operserv");
  
  osu = operserv_suser();
  
  suser_add_cmd(osu, "HOSTRULE", os_hostrule, 
    HOSTRULE_SUMMARY, HOSTRULE_HELP);  
    
  /* Add user events */
  irc_AddEvent(ET_NEW_USER, ev_os_hostrule_new_user); /* new user */        
  irc_AddEvent(ET_QUIT, ev_os_hostrule_quit); /* user quit */
  irc_AddEvent(ET_KILL, ev_os_hostrule_kill); /* user kill */  
  hostrules_init();  /* read hostrules from db */

  /* Create hostserv user */
  hrsu.u = irc_CreateLocalUser(HR_Nick, HR_Username, HR_Host, HR_Host,
      HR_Info,"+ro");

  mod_add_event_action(e_expire, (ActionHandler) ev_os_hostrule_expire);
  ev_os_hostrule_expire(NULL, NULL);
  return 0;
}

void
mod_unload(void)
{
  suser_del_mod_cmds(osu, &mod_info);
  irc_DelEvent(ET_NEW_USER, ev_os_hostrule_new_user);
  irc_DelEvent(ET_QUIT, ev_os_hostrule_quit);
  irc_DelEvent(ET_KILL, ev_os_hostrule_kill);
}
 
/* s = service the command was sent to
   u = user the command was sent from */
void os_hostrule(IRC_User *s, IRC_User *u)
{
  char *cmd;
  char *typestr;
  char *hostmask;
  char *msg;
  int rtype;
  int rtt;
  int param;
  u_int32_t id;
  HostRule* hr;
  struct tm *tm;
  char buf[128];
  time_t t_expire;
  cmd = strtok(NULL, " ");
  typestr = strtok(NULL, " ");
  hostmask = strtok(NULL, " ");    
  param = 0;

  if(IsNull(cmd) || *cmd=='\0' || IsNull(typestr)|| *typestr=='\0')
  {
    send_lang(u, s, HOSTRULE_SYNTAX);  
    return ;
  }

  rtt = string2rtt(typestr);
  if(rtt<0)
    {
      send_lang(u, s, INVALID_RULE_TYPE);
      return;
    }
  rtype = ruletypes[rtt].rtype;
    
  msg = strtok(NULL, "");        

  if(msg)
  {
    if(rtype < 512)
      param = strip_reason(&msg);
    else
    {
      param = atoi(msg);
      msg = strchr(msg, ' ');
      if(msg)
        ++msg;
    }
  }

  if(strcasecmp(cmd,"ADD") == 0)
   {
    if(IsNull(hostmask) || IsNull(msg))
      {
        send_lang(u, s, HOSTRULE_ADD_SYNTAX);  
        return ;
      }      
     if(((rtype>511) && (param < 1)) || (param == 0))
       {
         if(rtype<512)
           param = DefaultHRTime;
         else
           {
             send_lang(u, s, HOSTRULE_MANDATORY_PARAM);
             return;
           }
       }
     id = add_hostrule(hostmask, rtype, param, msg);
     send_lang(u, s, RULE_ADDED, typestr, id);
     if(ruletypes[rtt].rtype == HR_AGLINE) /* special case for aglines */
        irc_Gline(hrsu.u, hrsu.u->nick, hostmask, param,
          msg ? msg : "");
   }
  else if(strcasecmp(cmd,"DEL") == 0)
    {
      if(IsNull(hostmask))
      {
         send_lang(u, s, HOSTRULE_DEL_SYNTAX);
         return ;
      }
      id = atoi(hostmask); 
      if(del_hostrule(rtype, id)==-1)
        send_lang(u, s, RULE_NOT_FOUND, typestr, id);
      else
        send_lang(u, s, RULE_X_X_DELETED, ruletypes[rtt].str, id);
    }
  else if(strcasecmp(cmd,"LIST") == 0)
    {
      ev_os_hostrule_expire(NULL, NULL);    /* expire first */
      send_lang(u, s, RULE_LIST_HEADER, ruletypes[rtt].str);
      hr = get_hostrule(rtype, 1); /* get first element */
      while(hr)
        {
          if(rtype<512)
            {
              t_expire = hr->t_create + hr->param;
              tm = localtime(&t_expire);
              strftime(buf, sizeof(buf), format_str(u, DATE_FORMAT), tm);
              send_lang(u, s, RULE_X, hr->id, hr->host, buf, hr->message);
             }
           else
             send_lang(u, s, RULE_X_PARAM, hr->id, hr->host, hr->param, hr->message);
          hr = get_hostrule(rtype, 0); /* get next element */
        }
      send_lang(u, s, RULE_LIST_TAIL);
    }
  else
   send_lang(u, s, HOSTRULE_SYNTAX);
}

/* handle a new user event */
void ev_os_hostrule_new_user(IRC_User* u, void *s)
{
  HostRule *hr;
  HostRecord *hrec;
  int i;

  /* check for SessionLimit */  
  hrec = add_to_session(u->realhost);
  
  hr = next_hostrule(u, -1);  
  while(hr)
    {
      /* we have a host rule that matchs, lookup the function and call it */
      i = 0;
      while(ruletypes[i].str && (ruletypes[i].rtype != hr->rtype))
        ++i;
      if(ruletypes[i].str) /* type was found */
        ((int (*)(IRC_User*, HostRule*)) ruletypes[i].action)(u,hr);
      hr = next_hostrule(u, 0);
    }
  
  if(hrec->limit && (hrec->count > hrec->limit)) /* we reached session limit for this host */
    {
      if((++hrec->limit_hits) < MaxSessionHits)
        {
          log_log(os_log, mod_info.name, "Killing %s , limit of connections, %d conns, max=%d",
            irc_UserMask(u), hrec->count, hrec->limit);
          irc_Kill(u, osu->u, 
            "Reached maximum number of connections for this host");          
          irc_AbortThisEvent(); /* user was deleted, we need to abort the event*/
        }
      else
        {
          char gmask[HOSTLEN+3];        
          snprintf(gmask, HOSTLEN+3, "*@%s", u->realhost);                  
          log_log(os_log, mod_info.name, "Glining  %s, limit of connections, %d conns, max=%d",
            gmask, hrec->count, hrec->limit);            
          irc_Gline(osu->u, osu->u->nick, gmask, SessionsGlineTime,
            "Reached maximum number of connections for this host");
        }
    }
}


int hr_message(IRC_User* user, HostRule* rule)
{
  if(irc_IsUMode(user, UMODE_IDENTIFIED)) /* nick was identified */
    return -1;
  if(rule->message)
    irc_SendMsg(user, hrsu.u, "%s", rule->message);
  return -1;
}

int hr_notice(IRC_User* user, HostRule* rule)
{
  if(irc_IsUMode(user, UMODE_IDENTIFIED)) /* nick was identified */
    return -1;
  if(rule->message)
    irc_SendNotice(user, hrsu.u, "%s", rule->message);
  return -1;
}

int hr_agline(IRC_User* user, HostRule* rule)
{
  int duration = irc_CurrentTime-(rule->t_create+rule->param);
  if(duration > 0)
    irc_Gline(hrsu.u, hrsu.u->nick, rule->host, duration,
      rule->message ? rule->message : "");
  return -1;
}

int hr_bot(IRC_User* user, HostRule* rule)
{
  HostRecord *hr;
  irc_SendNotice(user, hrsu.u, "Set Bot Reason: %s", rule->message);  
  irc_SvsMode(user, hrsu.u, "+B");
  hr = find_session(user->realhost);
  if(hr) /* change the host limit */
    hr->limit = rule->param;
  return -1;
}

int hr_zombie(IRC_User* user, HostRule* rule)
{
  irc_SvsMode(user, hrsu.u, "+z");
  return -1;
}

void ev_os_hostrule_quit(IRC_User* u, char *lastquit)
{
  del_from_session(u->realhost);
}

void ev_os_hostrule_kill(IRC_User* u, IRC_User *killer)
{
  del_from_session(u->realhost);
}  
/* string2rtt
   RETURNS: index for the rule type
   		-1 if not found
*/
int string2rtt(char *str)
{
  int i;
  i = 0;
  while(ruletypes[i].str && strcasecmp(ruletypes[i].str, str))
    ++i;
  if(ruletypes[i].str)
    return i;
  else 
    return -1;
}

/* returns a session in memory */
HostRecord* find_session(char *host)
{
  HostRecord* hr;
  u_int32_t hashv;

  hashv = hash_host(host);
  hr = sessionTable[hashv];
  while(hr && strcasecmp(hr->host, host) != 0)
      hr = hr->hnext;

  return hr;
}
                  
/* adds a session to the hash table */
HostRecord* add_to_session(char *host)
{
  HostRecord* hr;
  u_int32_t hashv;
  
  hr = find_session(host); /* check if its already in mem */
  if(IsNull(hr))
    {
      hr = CreateHR(host); /* check if we have a db entry for this host*/
      if((hr->limit == -1))
        hr->limit = DefSessionLimit;
      hashv = hash_host(host);
      hr->hnext = sessionTable[hashv];
      strncpy(hr->host, host, HOSTLEN);
      sessionTable[hashv] = hr;
    }
  hr->count++;
  return hr;
}

void del_from_session(char *host)
{
  HostRecord *hr, *prev;
  int hashv;
  
  hashv = hash_host(host);
  hr = sessionTable[hashv];
  prev = NULL;
  
  if(IsNull(hr))
    return;
    
  while(hr && strcasecmp(hr->host, host) != 0)
    {
      prev = hr;    
      hr = hr->hnext;
    }
        
  if(IsNull(hr))
    return; /* not found */
    
  if(--(hr->count)==0) /* zero count, remove record from mem */
    {
      if(prev)
        prev->hnext= hr->hnext;
      else
        sessionTable[hashv] = hr->hnext;
      free(hr);
    }
}

unsigned int hash_host(const char* host)
{
  unsigned int h = 0;
                                                                                
  while (*host)
    {
      h = (h << 4) - (h + (unsigned char)ToLower(*host++));
    }
                                                                                
  return(h & (HOST_HSIZE - 1));
}

/* add an host rule 
  returns: id of the added rule
  */
int add_hostrule(char *host, int rtype, int param, char *message)
{
  HostRule* hr;
  
  hr = malloc(sizeof(HostRule));
  SDUP(hr->host, host);
  hr->rtype = rtype;
  hr->t_create = irc_CurrentTime;
  hr->param = param;
  SDUP(hr->message, message);
  hr->hnext = RulesList;
  RulesList = hr;
  db_insert_hostrule(hr);
  hr->id = sql_last_id();
  return hr->id;
}

/* del an host rule 
returns: 
	-1 if host not found
        0 on sucess	
  */
int del_hostrule(int rtype, int id)
{
 HostRule *hr, *old_hr;
 
 old_hr = NULL;
 hr = RulesList;
 while (hr && (hr->rtype != rtype || hr->id != id ))
   {
     old_hr = hr;
     hr = hr->hnext;     
   }
 if(hr) /* it was found */
   {
     if(old_hr)
       old_hr->hnext = hr->hnext;
     else
       RulesList = hr->hnext;
       
     FREE(hr->message);
     free(hr->host);
     free(hr);
     db_mysql_delete_hosrtule(id);
     return 0;
   }
 return -1;
}

/* returns next host rule that mathes  */
HostRule* next_hostrule(IRC_User *u, int start)
{
  static HostRule *hr; 
  if(start)
    hr = RulesList;
  else
    hr = hr->hnext;
  while(hr)
    {
     if((match(hr->host, u->realhost) == 1) ||
         (match(hr->host, u->publichost) == 1) 
        )
       {
         return hr; 
       }
     hr = hr->hnext;
    }
  return NULL;
}

/* return host rule */
HostRule* get_hostrule(int rtype, int first)
{
  static HostRule* cursor;
  if(first)
    {
      cursor = RulesList;
    }
  else
    cursor = cursor->hnext;
    
  while(cursor && 
    ((cursor->rtype<512 && cursor->t_create+cursor->param<irc_CurrentTime) || 
    cursor->rtype != rtype)) /* skip if it is not the correct type */
    cursor = cursor->hnext;
    
  return cursor;
}

void hostrules_init(void)
{
  RulesList = NULL;
  db_read_hostrules(&RulesList);  
}

/* Remove snid from nicks table on drop */
int db_insert_hostrule(HostRule* hr)
{
  return sql_execute("INSERT INTO os_hostrule (id, host, rtype, t_create, param, message) "\
    "VALUES (0, %s, %d, FROM_UNIXTIME(%d), %d, %s)",\
    sql_str(hr->host), hr->rtype,  (int) hr->t_create, (int)hr->param, sql_str(hr->message));
}

/* Remove hostrule from db */
int db_mysql_delete_hosrtule(int id)
{
    return sql_execute("DELETE FROM os_hostrule WHERE id=%d", id);    
}

int db_read_hostrules(HostRule** hlist)
{
  HostRule* hr;
  MYSQL_RES* res;

  res = sql_query("SELECT id,rtype,host,UNIX_TIMESTAMP(t_create), param, message FROM os_hostrule ORDER by id DESC");
  
  if(res == NULL)
    return 0;
    
  while(sql_next_row(res))
    {
      hr = malloc(sizeof(HostRule));
      hr->id = atoi(sql_field(0));
      hr->rtype = atoi(sql_field(1));
      hr->host = strdup(sql_field(2));
      hr->t_create = atoi(sql_field(3));
      hr->param = atoi(sql_field(4));
      hr->message = strdup(sql_field(5));
      hr->hnext= *hlist;
      *hlist = hr;      
    } while(sql_next_row(res));
  
  sql_free(res);  
  return 1;
}

HostRecord* CreateHR(char *host)
{
  HostRecord* hr;
  hr = malloc(sizeof(HostRecord));
  bzero(hr,sizeof(HostRecord));
  hr->limit = -1;
  return hr;
}

/* check for expired hostrules */
int ev_os_hostrule_expire(void* dummy1, void* dummy2)
{
  HostRule *hr, *old_hr, *next_hr;
  u_int32_t id;  
  
  old_hr = NULL;
  hr = RulesList;
  while (hr)
  {
    
    if(hr->rtype < 512 && /* check if can expire */
      hr->t_create + hr->param <= irc_CurrentTime )
    {
      log_log(os_log, mod_info.name, "Expiring hostrule %d, host=%s",
        hr->id, hr->host);
      id = hr->id;
      next_hr = hr->hnext;    
      if(old_hr)
        old_hr->hnext = next_hr;
      else
        RulesList = next_hr;  
      FREE(hr->message);
      free(hr->host);
      free(hr);
      db_mysql_delete_hosrtule(id);
      hr = next_hr;
    }
    else
    {
      old_hr = hr;
      hr = hr->hnext;  
    }
  }
  
  return 0;
}

/* End of module */
const int MVERSION = MODULES_VERSION;
