/**********************************************************************
 * PTlink IRC Services is (C) CopyRight PTlink IRC Software 1999-2005 *
 *                     http://software.pt-link.net                    *
 * This program is distributed under GNU Public License               *
 * Please read the file COPYING for copyright information.            
 **********************************************************************

  Description: operserv sendpass command

 *  $Id: os_sendpass.c,v 1.1.1.1 2005/08/27 15:44:27 jpinto Exp $
*/

#include "stdinc.h"
#include "dconf.h"
#include "modules.h"
#include "strhand.h"
#include "ircservice.h"
#include "suser.h"
#include "my_sql.h"
#include "nsmacros.h"
#include "lang.h"
#include "ns_group.h"   /* is_sadmin */
#include "log.h"
#include "encrypt.h"
#include "lang/nscommon.lh"
#include "lang/common.lh"
#include "lang/os_sendpass.lh"

#define PASSLEN	10	/* reset password length */

SVS_Module mod_info =
 /* module, version, description */
{"os_sendpass", "2.0",  "operserv sendpass module" };

/* Change Log
  2.0 - 0000265: remove nickserv cache system
*/

/** functions/events we require **/
/* void (*FunctionPointer)(void);*/
ServiceUser* (*operserv_suser)(void);

Module_Function mod_requires[] = 
{
  MOD_FUNC(operserv_suser)
  MOD_FUNC(is_sadmin)
  MOD_FUNC(is_sroot)
  /* MOD_FUNC(FunctionName) 
  ...
  */
  {NULL}
};

/** functions/events we provide **/
/* void my_function(void); */

Module_Function mod_provides[] = 
{
  /* MOD_FUNC(FunctionName) 
  ...
  */
  {NULL}
};

/** Internal functions declaration **/
int mod_load(char *param);
int mod_unload(char *param);
/* void internal_function(void); */
void os_sendpass(IRC_User *s, IRC_User *u);


/* Local dconf settings */
char *EmailFrom = NULL;
char *EmailFromName = NULL;

/* Get configuration we need locally */
int get_local_dconf(void)
{
  DCONF_GET_STRVALUE(EmailFrom)
  DCONF_GET_STRVALUE(EmailFromName)
  return 0;
}
      
/** Local variables **/
/* int my_local_variable; */
ServiceUser *osu = NULL;
int os_log = 0;
    
/** load code **/
int mod_load(char *param)
{

#ifndef SENDMAIL
  fprintf(stderr, "sendmail binary path was not found, you can't use this module");
  return -1;
#else
  os_log = log_handle("operserv");  
  osu = operserv_suser();  
  get_local_dconf();    
  suser_add_cmd(osu, "SENDPASS", (void*) os_sendpass, SENDPASS_SUMMARY, SENDPASS_HELP);    
  return 0;  
#endif  

}
    
/** unload code **/
int mod_unload(char *param)
{
  suser_del_mod_cmds(osu, &mod_info);
  return 0;
}
    
/** internal functions implementation starts here **/
void os_sendpass(IRC_User *s, IRC_User *u)
{
  u_int32_t source_snid;
  u_int32_t snid;
  char *target;
  
  /* status validation */  
  CHECK_IF_IDENTIFIED_NICK
  /* syntax validation */
  target = strtok(NULL, " ");
  
  if(!irc_IsUMode(u, UMODE_OPER)) /* extra security */
    return;
  else if(IsNull(target))
    send_lang(u, s, SENDPASS_SYNTAX);
  else if( (snid = nick2snid(target)) == 0 )
    send_lang(u, s, NICK_X_NOT_REGISTERED, target);
  /* sub-command */
  else if(is_sadmin(snid) || is_sroot(snid))
    {
      log_log(os_log, mod_info.name, "Nick %s trying SENDPASS on sadmin/soper %s",
        s->nick, target);
      irc_SendSanotice(s, "Nick %s trying SENDPASS on sadmin/soper %s",
        s->nick, target);
    }
  else
    {
#if 0    
      FILE *p;
      char sendmail[PATH_MAX];
      /* char buf[BUFSIZE];*/
      char pbuf[PASSLEN+1];
#ifdef SENDMAIL		  
      snprintf(sendmail, sizeof(sendmail), "%s %s", 
        SENDMAIL, nr->email);
#endif      
      if (!(p = popen(sendmail, "w")))
        send_lang(u, s, MAIL_LATER);
      else
        {
          /* generate a new password */
          rand_string(pbuf, PASSLEN, PASSLEN);
          pbuf[2] = '0'+ (random() % 10);
          memcpy(nr->pass, encrypted_password(pbuf), 16);
          UpdateNR(nr);
    	  fprintf(p, "From: \"%s\" <%s>\n", 
    	    EmailFromName, EmailFrom);
          fprintf(p, "To: \"%s\" <%s>\n", nr->nick, nr->email);
          fprintf(p, "Subject: Nick Password\n");
          fprintf(p, "\n");
          fprintf(p, "%s", lang_str_l(nr->lang, SENDPASS_X_X, nr->nick, pbuf));
          fprintf(p, "\n");
          fprintf(p, "\n.\n");
          pclose(p);      
          memset(pbuf, 0, PASSLEN);
          send_lang(u, s, SENDPASS_X_SENT_X, nr->nick, nr->email);
          log_log(os_log, mod_info.name, "SENDPASS for %s requested by %s", 
            nr->nick, u->nick);
          irc_SendSanotice(s, "SENDPASS for %s requested by %s",
            nr->nick, u->nick);
        }
#endif        
    }
}

/* End of module */
const int MVERSION = MODULES_VERSION;
