/**********************************************************************
 * PTlink IRC Services is (C) CopyRight PTlink IRC Software 1999-2005 *
 *                     http://software.pt-link.net                    *
 * This program is distributed under GNU Public License               *
 * Please read the file COPYING for copyright information.            *
 **********************************************************************

  Description: email sending functions

 *  $Id: email.c,v 1.1.1.1 2005/08/27 15:43:39 jpinto Exp $
*/

/*
 * Sends a given email
 */

#include "stdinc.h"
#include "email.h"
#include "strhand.h" /* FREE() */
#include "s_log.h" /* errlog() */
#include "path.h" /* ETCPATH */
#include "lang.h" /* MAX_LANGS */

static int symcount = 0; /* count of stored symbols */
static char* symbol_names[32];
static char* symbol_data[32];

/* free memory allocated for the email templates */
void email_free(char **emails)
{
  int i;
  for(i=0; i<MAX_LANGS;++i)
    free(emails[i]);    
}

/* load an email template from file */
int email_load(char *name, char **emails)
{
  int i;
  char *tld = NULL;
  char buf[2048];
  FILE *f;
  
  for(i=0; i<MAX_LANGS;++i)
  {
    index2lang(i, tld);
    if(tld==NULL)
      continue;
    snprintf(buf, sizeof(buf), "%s/mails/%s.%s", ETCPATH, name, tld);
    if (!(f = fopen(buf, "r")))
      {
        errlog(L_ERROR, "Failed to load email %s\n", buf);
        return -1;
      }
  
    bzero(buf, sizeof(buf));
    fread(buf, 1, sizeof(buf)-1, f);    
    emails[i] = strdup(buf);    
    fclose(f);
  }
  return 0;
}

/*
 * sens an email, replacing all variables (symbols)
 */
int email_send(char *msg)
{
  char buf[2048];
  char sendmail[PATH_MAX];
  int len;
  char *s, *t;  
  FILE *p;

#ifdef SENDMAIL
  snprintf(sendmail, sizeof(sendmail), "%s -f%s %s",
    SENDMAIL,  email_symbol("from"), email_symbol("email"));
#else
  slog(L_WARN, "Trying to send auth without SENDMAIL detected");
  return -1;
#endif

  len = strlen(msg);

  if(len > sizeof(buf))
    {
      errlog(L_ERROR, "Trying to send email too big!");
      return -1;
    }
    
  /* build new buffer applying symbols replacement */
  s = msg;
  t = buf;
  while(*s && ( t-buf < sizeof(buf) ))
  {
    if(*s == '%')
    {
      char *symbol;
      ++s;
      symbol = s;
      while(*s && *s !='%')
        ++s;
      if(*s == '%')
        {
          *s = '\0';
          t += snprintf(t, sizeof(buf)-(t-buf), "%s", email_symbol(symbol));
          *s = '%';
        }
    } 
    else
      *(t++) = *s;
    ++s;
  }
  *t = '\0';

  if (!(p = popen(sendmail, "w")))
    {
      errlog(L_ERROR, "ERROR, could not popen on email_send()");
      return -1;
    }
  fprintf(p, "%s", buf);
  pclose(p);

  return 0;
}

/*
 * initialize the symbols list
 */
void email_init_symbols(void)
{
  symcount = 0;
}

/*
 * adds a symbol to the symbols list
 */
void email_add_symbol(char *name, char* value)
{
  symbol_names[symcount] = name;
  symbol_data[symcount] = value;
  ++symcount;
}

/* 
 * returns the value of a symbol
 */
char* email_symbol(char *name)
{
  int i;
  for(i = 0; i < symcount; ++i)
  {
    if(strcmp(name, symbol_names[i]) == 0)
      return symbol_data[i];
  }
  return "<undefined>";
}
