/**********************************************************************
 * PTlink IRC Services is (C) CopyRight PTlink IRC Software 1999-2005 *
 *                     http://software.pt-link.net                    *
 * This program is distributed under GNU Public License               *
 * Please read the file COPYING for copyright information.            *
 **********************************************************************  

  Description: mysql functions

 *  $Id: mysql.c,v 1.1.1.1 2005/08/27 15:43:46 jpinto Exp $
*/

#include "stdinc.h"
#include "path.h"
#include "ircservice.h"
#include "s_log.h"
#include "log.h"
#include "chanrecord.h"
#include "ircsvs.h"
#include "dconf.h"
#include "my_sql.h"
#include "sqlb.h"
#include "hash.h"
#include "strhand.h"

extern int sql_debug;

/* DB_* are external, should be loaded from dconf */
char* DB_Host = NULL;
char* DB_Name = NULL;
char* DB_User = NULL;
char* DB_Pass = NULL;

int mysql_log = 0;

int sql_close();
int db_mysql_query(char *sql);

MYSQL *mysql;                   /* MySQL Handler */
MYSQL_ROW my_row;

#define CR_SELECT_FIELDS \
      "scid, name, url, email, founder, successor," \
      "last_topic, last_topic_setter, t_ltopic, t_reg,  t_last_use," \
      "mlock, status, flags, entrymsg, cdesc, t_maxusers," \
      "maxusers"

#define CR_UPDATE_PARAM \
      "name=%s, url=%s, email=%s, founder=%s, successor=%s," \
      "last_topic=%s, last_topic_setter=%s, t_ltopic=%d, t_reg=%d, t_last_use=%d," \
      "mlock=%s, status=%d, flags=%d, entrymsg=%s, cdesc=%s, t_maxusers=%d, maxusers=%d " \
      "WHERE scid=%d", \
      sql_str(irc_lower(cr->name)), sql_str(cr->url), sql_str(cr->email), founder, successor, \
      sql_str(cr->last_topic), sql_str(cr->last_topic_setter), (int) cr->t_ltopic, (int) cr->t_reg, (int) cr->t_last_use,\
      sql_str(cr->mlock), cr->status, cr->flags, sql_str(cr->entrymsg), sql_str(cr->cdesc), (int) cr->t_maxusers, cr->maxusers, \
      cr->scid \

void db_mysql_error(int severity, char *msg)
{
    static char buf[512];

    if (mysql_error(mysql)) {
        snprintf(buf, sizeof(buf), "MySQL %s %s: %s", msg,
                 severity == MYSQL_WARNING ? "warning" : "error",
                 mysql_error(mysql));
    } else {
        snprintf(buf, sizeof(buf), "MySQL %s %s", msg,
                 severity == MYSQL_WARNING ? "warning" : "error");
    }

    log_log(mysql_log, "ERROR", "%s", buf);
    fprintf(stderr, "%s\n", buf);

    if (severity == MYSQL_ERROR) {
        log_log(mysql_log, "FATAL", "MySQL FATAL error... aborting.");
        slog(L_ERROR, "MySQL FATAL error... aborting.");
        exit(0);
    }
}


const char *sql_error(void)
{
  return mysql_error(mysql);
}

/*************************************************************************/


int db_mysql_open()
{
  mysql = mysql_init(NULL);
  
  if ((!mysql_real_connect
      (mysql, DB_Host, DB_User, DB_Pass, DB_Name, 0, NULL, 0))) 
      {
        printf("MySQL ErrNo: %d\n", mysql_errno(mysql)); 
        log_log(mysql_log, "ERROR", "%s", mysql_error(mysql));            
        slog(L_ERROR,"Cant connect to MySQL: %s\n", mysql_error(mysql));
        return -1;
      }
  slog(L_INFO,"MySQL connected to %s", DB_Host);
  log_log(mysql_log, "Connect", "MySQL connected to %s", DB_Host);

  return 1;  
}

/*************************************************************************/

int db_mysql_query(char *sql)
{

    int result, lcv;
    /* slog(L_INFO,"%s", sql); */
    result = mysql_query(mysql, sql);

    if (result) {
        switch (mysql_errno(mysql)) {
        case CR_SERVER_GONE_ERROR:
        case CR_SERVER_LOST:
            /* Reconnect -> 5 tries (need to move to config file) */
            for (lcv = 0; lcv < 5; lcv++) {
                if (db_mysql_open()>0) {
                    result = mysql_query(mysql, sql);
                    return (result);
                }
                sleep(1);
            }

            /* If we get here, we could not connect. */
            slog(L_ERROR, "Unable to reconnect to database: %s\n",
                       mysql_error(mysql));
            db_mysql_error(MYSQL_ERROR, "connect");
            exit(-5);
            /* Never reached. */
            break;

        default:
            /* Unhandled error. */
            return (result);
        }
    }

    return (0);

}

/**
   returns a sql safe string (between "'") removing "\" and "'"
   it will also remove those chars from the parameter string;
*/
char* sql_str(char *str)
{
  static char bufs[20][512]; /* we can keep 20 strings */
  static int stri = 0; /* current string index */
  char* cbuf; /* current buffer */
  char* c; /* position on str */
  int i;
  
  if(IsNull(str))
    return "NULL";
  cbuf = bufs[stri++];
  
  if(stri>19) /* reached end of array */
    stri = 0;
  
  cbuf[0] = '\''; /* start the string */
  i = 1;
  c = str;
  while(*c && (i<507)) /* 511 - (2 for "\" + char), 2 for string end */
    {
      /* prefix with "\" */
      if((*c == '\'') || (*c == '\\'))      
        cbuf[i++] = '\\';
        
      cbuf[i++] = *(c++);
    }  
  cbuf[i] = '\''; /* close the string */
  cbuf[i+1] = '\0';  
  
  return cbuf;
}


/*************************************************************************/

int sql_close()
{
    mysql_close(mysql);
    return 1;
}

/* */
u_int32_t sql_execute(char *fmt, ...)
{
  static char buf[4096];
  va_list args;
  
  
  va_start(args, fmt);
  vsnprintf(buf, sizeof(buf), fmt, args);
  va_end(args);

  if(sql_debug)
    stdlog(L_INFO, "%s", buf);    

  if (db_mysql_query(buf)) 
    {
      /* slog(L_ERROR, "Can't create sql query: %s", buf);*/
      log_log(mysql_log, "ERROR", "sql_execute() %s", buf);
      db_mysql_error(MYSQL_WARNING, "query");
      return 0;
    }    
    
  if(strncasecmp(buf,"INSERT",6) == 0)
    {
      u_int32_t id;
      id = mysql_insert_id(mysql);
      if(id == 0) /* only non tables without id return 0 */
        return mysql_affected_rows(mysql);
      return id;
    }

  return  mysql_affected_rows(mysql);  
}

/* */
MYSQL_RES* sql_query(char *fmt, ...)
{
  static char buf[4096];
  va_list args;
  
  va_start(args, fmt);
  vsnprintf(buf, sizeof(buf), fmt, args);
  va_end(args);

  if(sql_debug)
    stdlog(L_INFO, "%s", buf);  
  if (db_mysql_query(buf)) 
    {
      log_log(mysql_log, "ERROR", "sql_query() %s", buf);
      /* slog(L_ERROR, "Can't create sql query: %s", buf); */
      db_mysql_error(MYSQL_WARNING, "query");
      return NULL;
    }
    
  return mysql_store_result(mysql);
}

/* */
int sql_singlequery(char *fmt, ...)
{
  static char buf[4096];
  va_list args;
  static MYSQL_RES* res = NULL;

  if(res)
    {
      sql_free(res);
      res = NULL;
    }
    
  va_start(args, fmt);
  vsnprintf(buf, sizeof(buf), fmt, args);
  va_end(args);

  if(sql_debug)
    stdlog(L_INFO, "%s", buf);  
    
  if (db_mysql_query(buf)) 
    {
      log_log(mysql_log, "ERROR:", "sql_fastquery() %s", buf);
      /* slog(L_ERROR, "Can't create sql query: %s", buf); */
      db_mysql_error(MYSQL_WARNING, "query");
      return 0;
    }
    
  res = mysql_store_result(mysql);
  if(res)
    {
      my_row = mysql_fetch_row(res);
      if(my_row)
        return 1;
    }
    
  return 0;
}

/* Moves to the next row */
MYSQL_ROW sql_next_row(MYSQL_RES* res)
{
  if(res == NULL)
    return NULL;
    
  my_row = mysql_fetch_row(res);
  return my_row;
}

void sql_free(MYSQL_RES* res)
{
  mysql_free_result(res);
}

char *sql_field(int i)
{
  if(my_row)
    return my_row[i];
  else
    return NULL;
}

u_int32_t sql_field_i(int i)
{
  if(my_row && my_row[i])
    return atoi(my_row[i]);
  else
    return 0;
}

MYSQL_RES* sql_use_result(void)
{
  return mysql_use_result(mysql);
}


u_int32_t sql_last_id(void)
{
  return mysql_insert_id(mysql);
}

/*************************************************************************/

/*
 * NickRecord functions *
 */

u_int32_t nick2snid(char *name)
{
  MYSQL_RES* mysql_res;    
  MYSQL_ROW row;
  char sqlcmd[MAX_SQL_BUF]; 
  u_int32_t snid;
  snprintf(sqlcmd, MAX_SQL_BUF,"SELECT snid FROM nickserv WHERE nick=%s", 
    sql_str(irc_lower_nick(name)));
  if (db_mysql_query(sqlcmd))
    {
      log_log(mysql_log, "ERROR:", "nick2snid() %s", sqlcmd);
      /*      
      slog(L_ERROR, "Can't create sql query: %s", sqlcmd);
      */
        db_mysql_error(MYSQL_WARNING, "query");
        
        return 0;
    }
  mysql_res = mysql_store_result(mysql);
  
  if(mysql_res == NULL)
    return 0;
  
  row = mysql_fetch_row(mysql_res);

  if(row == NULL)
    snid = 0;
  else 
    snid = atoi(row[0]);

  mysql_free_result(mysql_res);
  return snid;
}

/* returns count of nicks with given email */
int reg_count_for_email(char *email)
{
  MYSQL_RES* mysql_res;    
  MYSQL_ROW row;
  char sqlcmd[MAX_SQL_BUF]; 
  int count = 0;

  snprintf(sqlcmd, MAX_SQL_BUF,
    "SELECT count(*) FROM nickserv WHERE email=%s", sql_str(email));
  if (db_mysql_query(sqlcmd))
    {
      log_log(mysql_log, "ERROR:", "email_count() %s", sqlcmd);
        db_mysql_error(MYSQL_WARNING, "query");
        return 0;
    }
  mysql_res = mysql_store_result(mysql);
  row = mysql_fetch_row(mysql_res);
  if(row)
    count = atoi(row[0]);
  mysql_free_result(mysql_res);            
  return count;
}

/*
 * ChanRecord functions *
 */ 

/* Insert a new chan record */
int db_mysql_insert_cr(ChanRecord* cr)
{
    char successor[16];
    char founder[16];
    sqlb_init("chanserv");
    sqlb_add_int("scid", cr->scid);
    sqlb_add_str("name", irc_lower(cr->name));
    sqlb_add_str("email", cr->email);
    if(cr->founder)
    {
      snprintf(founder, sizeof(founder), "%d", cr->founder);
      sqlb_add_str("founder", founder);
    }
    else
      sqlb_add_str("founder", NULL);
    if(cr->successor)
    {    
      snprintf(successor, sizeof(successor), "%d", cr->successor);
      sqlb_add_str("successor", successor);
    }
    else
      sqlb_add_str("successor", NULL);
    sqlb_add_str("last_topic", cr->last_topic);
    sqlb_add_str("last_topic_setter", cr->last_topic_setter);
    sqlb_add_int("t_ltopic", cr->t_ltopic);
    sqlb_add_int("t_reg", cr->t_reg);
    sqlb_add_int("t_last_use", cr->t_last_use);
    sqlb_add_str("mlock", cr->mlock);
    sqlb_add_int("status", cr->status);
    sqlb_add_int("flags", cr->flags);
    sqlb_add_str("entrymsg", cr->entrymsg);
    sqlb_add_str("cdesc", cr->cdesc);
    sqlb_add_int("t_maxusers", cr->t_maxusers);
    sqlb_add_int("maxusers", cr->maxusers);

    if (db_mysql_query(sqlb_insert())) {
        log_log(mysql_log, "ERROR:", "insert_cr() %s", sqlb_insert());
        db_mysql_error(MYSQL_WARNING, "query");
        return -1;
    }
    cr->scid = mysql_insert_id(mysql);
    return cr->scid;
}

ChanRecord* db_mysql_get_cr(u_int32_t scid)
{
  MYSQL_RES* mysql_res;    
  MYSQL_ROW row;
  ChanRecord *cr;  
  char sqlcmd[MAX_SQL_BUF]; 
  
  snprintf(sqlcmd, MAX_SQL_BUF,
    "SELECT "
    CR_SELECT_FIELDS
    " FROM chanserv WHERE scid=%d", scid);
          
  if (db_mysql_query(sqlcmd)) 
    {
      log_log(mysql_log, "ERROR:", "get_cr() %s", sqlcmd);    
/*      
      slog(L_ERROR, "Can't create sql query: %s", sqlcmd);
*/      
        db_mysql_error(MYSQL_WARNING, "query");
      return NULL;
    }
  mysql_res = mysql_store_result(mysql);
  row = mysql_fetch_row(mysql_res);  
  if(IsNull(row))
    {
      mysql_free_result(mysql_res);     
      return NULL;
    }
        
 /* we have data, lets process it */
  cr = CreateCR(row[1]);
  cr->scid = atoi(row[0]);   
  SDUP(cr->url, row[2]); 
  SDUP(cr->email, row[3]);     
  if(row[4])
    cr->founder = atoi(row[4]);
  else
    cr->founder = 0;
  if(row[5])
    cr->successor = atoi(row[5]);
  else
    cr->successor = 0;
  SDUP(cr->last_topic, row[6]);
  SDUP(cr->last_topic_setter, row[7]);
  cr->t_ltopic = atoi(row[8]);
  cr->t_reg = atoi(row[9]);
  cr->t_last_use = atoi(row[10]);
  SDUP(cr->mlock, row[11]);
  cr->status = atoi(row[12]);
  cr->flags = atoi(row[13]);
  SDUP(cr->entrymsg, row[14]);
  SDUP(cr->cdesc, row[15]);    
  cr->t_maxusers = atoi(row[16]);
  cr->maxusers = atoi(row[17]);
  
  mysql_free_result(mysql_res); 
  return cr;
}

/* update chan record */
int db_mysql_update_cr(ChanRecord* cr)
{
    char sqlcmd[MAX_SQL_BUF];
    char successor[16];
    char founder[16];
    
    if(cr->successor)
      snprintf(successor, sizeof(successor), "'%d'", cr->successor);
    else
      strcpy(successor, "NULL");

    if(cr->founder)
      snprintf(founder, sizeof(founder), "'%d'", cr->founder);
    else
      strcpy(founder, "NULL");      
    
    snprintf(sqlcmd, MAX_SQL_BUF,
      "UPDATE chanserv SET "
      CR_UPDATE_PARAM
      );

    if (db_mysql_query(sqlcmd)) {
        log_log(mysql_log, "ERROR:", "update_cr() %s", sqlcmd);    
/*        
        slog(L_ERROR, "Can't create sql query: %s", sqlcmd);
*/        
        db_mysql_error(MYSQL_WARNING, "query");
        return 0;
    }
    return 1;
}

u_int32_t db_mysql_chan2scid(char *name)
{
  MYSQL_RES* mysql_res;
  MYSQL_ROW row;
  char sqlcmd[MAX_SQL_BUF]; 
  u_int32_t scid;
  snprintf(sqlcmd, MAX_SQL_BUF,"SELECT scid, name FROM chanserv WHERE name=%s", 
    sql_str(irc_lower(name)));
  if (db_mysql_query(sqlcmd))
    {
      log_log(mysql_log, "ERROR:", "chan2scid() %s", sqlcmd);    
      /*      
      slog(L_ERROR, "Can't create sql query: %s", sqlcmd);
      */      
        db_mysql_error(MYSQL_WARNING, "query");
        return 0;
    }
  mysql_res = mysql_store_result(mysql);
  
  if(mysql_res == NULL)
    return 0;
    
  while((row = mysql_fetch_row(mysql_res)) && irccmp(row[1], name)) ;
  
  if(row == NULL)
    scid = 0;
  else
    scid = atoi(row[0]);
    
  mysql_free_result(mysql_res);
  return scid;
}

/*
  Execute a sql command from file 
*/
int mysql_from_file(char *fn)
{
  FILE *f;
  char buf[32000];

  char *sql;
  int res;
  
  bzero(buf, 32000);  
  f = fopen(fn,"rt");  
  if(IsNull(f))
    {
      stdlog(L_ERROR, "Could not open %s \n", fn); 
      return -1;
    }
  fread(buf, sizeof(buf), 1, f);
  sql = strtok(buf,";");
  while(sql && strlen(sql)>10)
    {    
      res = mysql_query(mysql, sql);
      if(res<0)
        {
          errlog(L_ERROR,"MySQL Error: %s", mysql_error(mysql));
          errlog(L_ERROR, "SQL: %s", sql);
          fclose(f);
          return -2;
        }
      sql = strtok(NULL, ";");            
    }
  fclose(f);
  return 0;
}

int sql_errno(void)
{
  return mysql_errno(mysql);
}


/**
 * check, install or upgrade tables for a module
 * returns:
 *	<0 : there was some error
 *	0  : tables are ok
 *	1  : tables were installed
 *	2  : tables were upgraded
 */
int sql_check_inst_upgrade(char *name, int version, void* upgrade_func) 
{
  char fn[256];
  int current_version = 0;
  MYSQL_RES* res;
  MYSQL_ROW row;
  int i;
  
  res = sql_query("SELECT MAX(version) FROM ircsvs_tables WHERE name='%s'", name);
  
  if((row = sql_next_row(res)) && row[0])
    current_version = atoi(row[0]);
    
  sql_free(res);
  
  if(current_version == 0)
    {
      stdlog(L_WARN, "Installing %s table(s)", name);
      log_log(mysql_log, "INSTALL", "Installing %s table(s)", name);
      snprintf(fn, sizeof(fn), "%s/%s.sql", SQLPATH, name);
      if(mysql_from_file(fn) < 0)
        {
          errlog(L_ERROR, "Error executing SQL from %s", fn);
          return -2;      
        }
      if(sql_execute("INSERT INTO ircsvs_tables "
          " VALUES('%s',%d, NOW())", name, version) == 0)
        {
          errlog(L_ERROR, "Unable to insert table version info!");
          return -3;
        }
      return 1;
    }
  else if(current_version < version)
    {
      stdlog(L_WARN, "Upgrading %s tables from version %d to %d",
        name, current_version, version);        
      for(i = current_version+1; i <= version; ++i)
        {
          int r;
          stdlog(L_WARN, "SQL Upgrade %s.%d", name, i);
          if(upgrade_func) /* we have an upgrade function to call */
            {
              r = ((int (*)(int, int)) upgrade_func)(i, 0);
              if(r < 0)
                {
                  errlog(L_ERROR, "Error %i from pre upgrade routine %i", r, i);
                  return -3;
                }
            }          
          snprintf(fn, sizeof(fn), "%s/%s.%d.sql", SQLPATH, name, i);
          if(mysql_from_file(fn) < 0)
            {
              errlog(L_ERROR, "Error executing SQL from %s", fn);
              return -2;      
            }
          if(upgrade_func) /* we have an upgrade function to call */
            {
              r = ((int (*)(int, int)) upgrade_func)(i, 1);
              if(r < 0)
                {
                  errlog(L_ERROR, "Error %i from post upgrade routine %i", r, i);
                  return -3;
                }
            }
          if(sql_execute("INSERT INTO ircsvs_tables "
            " VALUES('%s',%d, NOW())", name, i) == 0)
            {
              errlog(L_ERROR, "Unable to insert table version info!");
              return -4;
            }
        }
      return 2;
    }
  else if(current_version > version)
    {
      errlog(L_WARN, "Module %s expects version %d, current version: %d",
        name, version, current_version); 
        return -4;
    }
  return 0;
}
